searchData={"items":[{"type":"behaviour","title":"OpcUA.Client","doc":"OPC UA Client API module.\n\nThis module provides functions for configuration, read/write nodes attributes and discovery of a OPC UA Client.\n\n`OpcUA.Client` is implemented as a `__using__` macro so that you can put it in any module,\nyou can initialize your Client manually (see `test/client_tests`) or by overwriting\n`configuration/1` and `monitored_items/1` to autoset the configuration and subscription items. It also helps you to\nhandle Client's \"subscription\" events (monitorItems) by overwriting `handle_subscription/2` callback.\n\nThe following example shows a module that takes its configuration from the environment (see `test/client_tests/terraform_test.exs`):\n\n```elixir\ndefmodule MyClient do\n  use OpcUA.Client\n\n  # Use the `init` function to configure your Client.\n  def init({parent_pid, 103} = _user_init_state, opc_ua_client_pid) do\n    %{parent_pid: parent_pid, opc_ua_client_pid: opc_ua_client_pid}\n  end\n\n  def configuration(_user_init_state), do: Application.get_env(:my_client, :configuration, [])\n  def monitored_items(_user_init_state), do: Application.get_env(:my_client, :monitored_items, [])\n\n  def handle_subscription_timeout(subscription_id, state) do\n    send(state.parent_pid, {:subscription_timeout, subscription_id})\n    state\n  end\n\n  def handle_deleted_subscription(subscription_id, state) do\n    send(state.parent_pid, {:subscription_delete, subscription_id})\n    state\n  end\n\n  def handle_monitored_data(changed_data_event, state) do\n    send(state.parent_pid, {:value_changed, changed_data_event})\n    state\n  end\n\n  def handle_deleted_monitored_item(subscription_id, monitored_id, state) do\n    send(state.parent_pid, {:item_deleted, {subscription_id, monitored_id}})\n    state\n  end\nend\n```\n\nBecause it is small a GenServer, it accepts the same [options](https://hexdocs.pm/elixir/GenServer.html#module-how-to-supervise) for supervision\nto configure the child spec and passes them along to `GenServer`:\n\n```elixir\ndefmodule MyModule do\n  use OpcUA.Client, restart: :transient, shutdown: 10_000\nend\n```","ref":"OpcUA.Client.html"},{"type":"function","title":"OpcUA.Client.add_monitored_item/2","doc":"Adds a monitored item used to request a server for notifications of each change of value in a specific node.\n  The following option must be filled:\n  * `:subscription_id` -> integer().\n  * `:monitored_item` -> %NodeId{}.","ref":"OpcUA.Client.html#add_monitored_item/2"},{"type":"function","title":"OpcUA.Client.add_subscription/2","doc":"Sends an OPC UA Server request to start subscription (to monitored items, events, etc).","ref":"OpcUA.Client.html#add_subscription/2"},{"type":"function","title":"OpcUA.Client.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"OpcUA.Client.html#child_spec/1"},{"type":"callback","title":"OpcUA.Client.configuration/1","doc":"Optional callback that gets the Server configuration and discovery connection parameters.","ref":"OpcUA.Client.html#c:configuration/1"},{"type":"function","title":"OpcUA.Client.connect_by_url/2","doc":"Connects the OPC UA Client by a url.\n  The following must be filled:\n  * `:url` -> binary().","ref":"OpcUA.Client.html#connect_by_url/2"},{"type":"function","title":"OpcUA.Client.connect_by_username/2","doc":"Connects the OPC UA Client by a url using a username and a password.\n  The following must be filled:\n  * `:url` -> binary().\n  * `:user` -> binary().\n  * `:password` -> binary().","ref":"OpcUA.Client.html#connect_by_username/2"},{"type":"function","title":"OpcUA.Client.connect_no_session/2","doc":"Connects the OPC UA Client by a url without a session.\n  The following must be filled:\n  * `:url` -> binary().","ref":"OpcUA.Client.html#connect_no_session/2"},{"type":"function","title":"OpcUA.Client.delete_monitored_item/2","doc":"Adds a monitored item used to request a server for notifications of each change of value in a specific node.\n  The following option must be filled:\n  * `:subscription_id` -> integer().\n  * `:monitored_item_id` -> integer().","ref":"OpcUA.Client.html#delete_monitored_item/2"},{"type":"function","title":"OpcUA.Client.delete_subscription/2","doc":"Sends an OPC UA Server request to delete a subscription.","ref":"OpcUA.Client.html#delete_subscription/2"},{"type":"function","title":"OpcUA.Client.disconnect/1","doc":"Disconnects the OPC UA Client.","ref":"OpcUA.Client.html#disconnect/1"},{"type":"function","title":"OpcUA.Client.find_servers/2","doc":"Finds Servers Connected to a Discovery Server.\n  The following must be filled:\n  * `:url` -> binary().","ref":"OpcUA.Client.html#find_servers/2"},{"type":"function","title":"OpcUA.Client.find_servers_on_network/2","doc":"Finds Servers Connected to a Discovery Server.\n  The following must be filled:\n  * `:url` -> binary().","ref":"OpcUA.Client.html#find_servers_on_network/2"},{"type":"function","title":"OpcUA.Client.get_config/1","doc":"Gets the OPC UA Client current Configuration.","ref":"OpcUA.Client.html#get_config/1"},{"type":"function","title":"OpcUA.Client.get_endpoints/2","doc":"Get endpoints from a OPC UA Server.\n  The following must be filled:\n  * `:url` -> binary().","ref":"OpcUA.Client.html#get_endpoints/2"},{"type":"function","title":"OpcUA.Client.get_state/1","doc":"Gets the state of the OPC UA Client.","ref":"OpcUA.Client.html#get_state/1"},{"type":"callback","title":"OpcUA.Client.handle_deleted_monitored_item/3","doc":"Optional callback that handles a deleted monitored items events.\n\nIt's first argument is the `subscription_id` of the subscription that the monitored\nitem belongs to. The second element is the 'monitored_item_id' which is an unique\nnumber asigned to a monitored item when its created.\n\nThe third argument it's the GenServer state (Parent process).","ref":"OpcUA.Client.html#c:handle_deleted_monitored_item/3"},{"type":"callback","title":"OpcUA.Client.handle_deleted_subscription/2","doc":"Optional callback that handles a subscriptions timeout events.\n\nIt's first argument is the `subscription_id` of the subscription.\n\nThe second argument it's the GenServer state (Parent process).","ref":"OpcUA.Client.html#c:handle_deleted_subscription/2"},{"type":"callback","title":"OpcUA.Client.handle_monitored_data/2","doc":"Optional callback that handles node values updates from a Client to a Server.\n\nIt's first argument is a tuple, in which its first element is the `subscription_id`\nof the subscription that the monitored item belongs to. the second element\nis the 'monitored_item_id' which is an unique number asigned to a monitored item when\nits created and the third element of the tuple is the new value of the monitored item.\n\nthe second argument it's the GenServer state (Parent process).","ref":"OpcUA.Client.html#c:handle_monitored_data/2"},{"type":"callback","title":"OpcUA.Client.handle_subscription_timeout/2","doc":"Optional callback that handles a subscriptions timeout events.\n\nIt's first argument is the `subscription_id` of the subscription.\n\nThe second argument it's the GenServer state (Parent process).","ref":"OpcUA.Client.html#c:handle_subscription_timeout/2"},{"type":"function","title":"OpcUA.Client.init/1","doc":"","ref":"OpcUA.Client.html#init/1"},{"type":"callback","title":"OpcUA.Client.monitored_items/1","doc":"","ref":"OpcUA.Client.html#c:monitored_items/1"},{"type":"function","title":"OpcUA.Client.read_node_access_level/2","doc":"Reads 'Access level' of a node in the server.","ref":"OpcUA.Client.html#read_node_access_level/2"},{"type":"function","title":"OpcUA.Client.read_node_array_dimensions/2","doc":"Reads 'array_dimensions' of a node in the server.","ref":"OpcUA.Client.html#read_node_array_dimensions/2"},{"type":"function","title":"OpcUA.Client.read_node_browse_name/2","doc":"Reads the browse name attribute of a node in the server.","ref":"OpcUA.Client.html#read_node_browse_name/2"},{"type":"function","title":"OpcUA.Client.read_node_contains_no_loops/2","doc":"Reads 'contains_no_loops' attribute of a node in the server.","ref":"OpcUA.Client.html#read_node_contains_no_loops/2"},{"type":"function","title":"OpcUA.Client.read_node_data_type/2","doc":"Reads 'data_type' attribute of a node in the server.","ref":"OpcUA.Client.html#read_node_data_type/2"},{"type":"function","title":"OpcUA.Client.read_node_description/2","doc":"Reads description attribute of a node in the server.","ref":"OpcUA.Client.html#read_node_description/2"},{"type":"function","title":"OpcUA.Client.read_node_display_name/2","doc":"Reads the display name attribute of a node in the server.","ref":"OpcUA.Client.html#read_node_display_name/2"},{"type":"function","title":"OpcUA.Client.read_node_event_notifier/2","doc":"Reads 'event_notifier' attribute of a node in the server.","ref":"OpcUA.Client.html#read_node_event_notifier/2"},{"type":"function","title":"OpcUA.Client.read_node_executable/2","doc":"Reads 'Executable' attribute of a node in the server.","ref":"OpcUA.Client.html#read_node_executable/2"},{"type":"function","title":"OpcUA.Client.read_node_historizing/2","doc":"Reads 'Historizing' attribute of a node in the server.","ref":"OpcUA.Client.html#read_node_historizing/2"},{"type":"function","title":"OpcUA.Client.read_node_inverse_name/2","doc":"Reads 'Inverse name' attribute of a node in the server.","ref":"OpcUA.Client.html#read_node_inverse_name/2"},{"type":"function","title":"OpcUA.Client.read_node_is_abstract/2","doc":"Reads 'Is Abstract' attribute of a node in the server.","ref":"OpcUA.Client.html#read_node_is_abstract/2"},{"type":"function","title":"OpcUA.Client.read_node_minimum_sampling_interval/2","doc":"Reads 'Minimum Sampling Interval level' of a node in the server.","ref":"OpcUA.Client.html#read_node_minimum_sampling_interval/2"},{"type":"function","title":"OpcUA.Client.read_node_node_class/2","doc":"Reads the node_class attribute of a node in the server.","ref":"OpcUA.Client.html#read_node_node_class/2"},{"type":"function","title":"OpcUA.Client.read_node_node_id/2","doc":"Reads the node_id attribute of a node in the server.","ref":"OpcUA.Client.html#read_node_node_id/2"},{"type":"function","title":"OpcUA.Client.read_node_symmetric/2","doc":"Reads 'Symmetric' attribute of a node in the server.","ref":"OpcUA.Client.html#read_node_symmetric/2"},{"type":"function","title":"OpcUA.Client.read_node_user_access_level/2","doc":"Reads 'user_access_level' attribute of a node in the server.","ref":"OpcUA.Client.html#read_node_user_access_level/2"},{"type":"function","title":"OpcUA.Client.read_node_user_executable/2","doc":"Reads 'user_executable' attribute of a node in the server.","ref":"OpcUA.Client.html#read_node_user_executable/2"},{"type":"function","title":"OpcUA.Client.read_node_user_write_mask/2","doc":"Reads 'user_write_mask' attribute of a node in the server.","ref":"OpcUA.Client.html#read_node_user_write_mask/2"},{"type":"function","title":"OpcUA.Client.read_node_value/3","doc":"Reads 'value' attribute of a node in the server.\nNote: If the value is an array you can search a scalar using `index` parameter.","ref":"OpcUA.Client.html#read_node_value/3"},{"type":"function","title":"OpcUA.Client.read_node_value_by_data_type/3","doc":"Reads 'Value' attribute (matching data type) of a node in the server.","ref":"OpcUA.Client.html#read_node_value_by_data_type/3"},{"type":"function","title":"OpcUA.Client.read_node_value_by_index/3","doc":"Reads 'value' attribute of a node in the server.\nNote: If the value is an array you can search a scalar using `index` parameter.","ref":"OpcUA.Client.html#read_node_value_by_index/3"},{"type":"function","title":"OpcUA.Client.read_node_value_rank/2","doc":"Reads 'Value Rank' of a node in the server.","ref":"OpcUA.Client.html#read_node_value_rank/2"},{"type":"function","title":"OpcUA.Client.read_node_write_mask/2","doc":"Reads 'Write Mask' attribute of a node in the server.","ref":"OpcUA.Client.html#read_node_write_mask/2"},{"type":"function","title":"OpcUA.Client.reset/1","doc":"Resets the OPC UA Client.","ref":"OpcUA.Client.html#reset/1"},{"type":"function","title":"OpcUA.Client.set_config/2","doc":"Sets the OPC UA Client configuration.","ref":"OpcUA.Client.html#set_config/2"},{"type":"function","title":"OpcUA.Client.set_config_with_certs/2","doc":"Sets the OPC UA Client configuration with all security policies for the given certificates.\n  The following must be filled:\n    * `:private_key` -> binary() or function().\n    * `:certificate` -> binary() or function().\n    * `:security_mode` -> interger().\n  NOTE: [none: 1, sign: 2, sign_and_encrypt: 3]","ref":"OpcUA.Client.html#set_config_with_certs/2"},{"type":"function","title":"OpcUA.Client.start_link/2","doc":"Starts up a OPC UA Client GenServer.","ref":"OpcUA.Client.html#start_link/2"},{"type":"function","title":"OpcUA.Client.stop/1","doc":"Stops a OPC UA Client GenServer.","ref":"OpcUA.Client.html#stop/1"},{"type":"function","title":"OpcUA.Client.write_node_access_level/3","doc":"Change 'Access level' of a node in the server.","ref":"OpcUA.Client.html#write_node_access_level/3"},{"type":"function","title":"OpcUA.Client.write_node_array_dimensions/3","doc":"Change 'Array Dimensions' of a node in the server.","ref":"OpcUA.Client.html#write_node_array_dimensions/3"},{"type":"function","title":"OpcUA.Client.write_node_blank_array/4","doc":"Creates a blank 'value array' attribute of a node in the server.\nNote: the array must match with 'value_rank' and 'array_dimensions' attribute.","ref":"OpcUA.Client.html#write_node_blank_array/4"},{"type":"function","title":"OpcUA.Client.write_node_browse_name/3","doc":"Change the browse name of a node in the server.","ref":"OpcUA.Client.html#write_node_browse_name/3"},{"type":"function","title":"OpcUA.Client.write_node_contains_no_loops/3","doc":"Change 'contains_no_loops' attribute of a node in the server.","ref":"OpcUA.Client.html#write_node_contains_no_loops/3"},{"type":"function","title":"OpcUA.Client.write_node_data_type/3","doc":"Change 'data_type' attribute of a node in the server.","ref":"OpcUA.Client.html#write_node_data_type/3"},{"type":"function","title":"OpcUA.Client.write_node_description/4","doc":"Change description attribute of a node in the server.","ref":"OpcUA.Client.html#write_node_description/4"},{"type":"function","title":"OpcUA.Client.write_node_display_name/4","doc":"Change the display name attribute of a node in the server.","ref":"OpcUA.Client.html#write_node_display_name/4"},{"type":"function","title":"OpcUA.Client.write_node_event_notifier/3","doc":"Change 'event_notifier' attribute of a node in the server.","ref":"OpcUA.Client.html#write_node_event_notifier/3"},{"type":"function","title":"OpcUA.Client.write_node_executable/3","doc":"Change 'Executable' attribute of a node in the server.","ref":"OpcUA.Client.html#write_node_executable/3"},{"type":"function","title":"OpcUA.Client.write_node_historizing/3","doc":"Change 'Historizing' attribute of a node in the server.","ref":"OpcUA.Client.html#write_node_historizing/3"},{"type":"function","title":"OpcUA.Client.write_node_inverse_name/4","doc":"Change 'Inverse name' attribute of a node in the server.","ref":"OpcUA.Client.html#write_node_inverse_name/4"},{"type":"function","title":"OpcUA.Client.write_node_is_abstract/3","doc":"Change 'Is Abstract' attribute of a node in the server.","ref":"OpcUA.Client.html#write_node_is_abstract/3"},{"type":"function","title":"OpcUA.Client.write_node_minimum_sampling_interval/3","doc":"Change 'Minimum Sampling Interval level' of a node in the server.","ref":"OpcUA.Client.html#write_node_minimum_sampling_interval/3"},{"type":"function","title":"OpcUA.Client.write_node_node_class/3","doc":"Change 'node_class' attribute of a node in the server.\n  Avalable value are:\n    UNSPECIFIED = 0,\n    OBJECT = 1,\n    VARIABLE = 2,\n    METHOD = 4,\n    OBJECTTYPE = 8,\n    VARIABLETYPE = 16,\n    REFERENCETYPE = 32,\n    DATATYPE = 64,\n    VIEW = 128,","ref":"OpcUA.Client.html#write_node_node_class/3"},{"type":"function","title":"OpcUA.Client.write_node_node_id/3","doc":"Change 'node_id' attribute of a node in the server.","ref":"OpcUA.Client.html#write_node_node_id/3"},{"type":"function","title":"OpcUA.Client.write_node_symmetric/3","doc":"Change 'symmetric' attribute of a node in the server.","ref":"OpcUA.Client.html#write_node_symmetric/3"},{"type":"function","title":"OpcUA.Client.write_node_user_access_level/3","doc":"Change 'user_access_level' attribute of a node in the server.","ref":"OpcUA.Client.html#write_node_user_access_level/3"},{"type":"function","title":"OpcUA.Client.write_node_user_executable/3","doc":"Change 'user_executable' attribute of a node in the server.","ref":"OpcUA.Client.html#write_node_user_executable/3"},{"type":"function","title":"OpcUA.Client.write_node_user_write_mask/3","doc":"Change 'user_write_mask' attribute of a node in the server.","ref":"OpcUA.Client.html#write_node_user_write_mask/3"},{"type":"function","title":"OpcUA.Client.write_node_value/5","doc":"Change 'Value' attribute of a node in the server.","ref":"OpcUA.Client.html#write_node_value/5"},{"type":"function","title":"OpcUA.Client.write_node_value_rank/3","doc":"Change 'Value rank' of a node in the server.\n\nThis attribute indicates whether the value attribute of the variable is an array and how many dimensions the array has.\nIt may have the following values:\n\nvalue_rank >= 1: the value is an array with the specified number of dimensions\nvalue_rank =  0: the value is an array with one or more dimensions\nvalue_rank = -1: the value is a scalar\nvalue_rank = -2: the value can be a scalar or an array with any number of dimensions\nvalue_rank = -3: the value can be a scalar or a one dimensional array","ref":"OpcUA.Client.html#write_node_value_rank/3"},{"type":"function","title":"OpcUA.Client.write_node_write_mask/3","doc":"Change 'Write Mask' attribute of a node in the server.","ref":"OpcUA.Client.html#write_node_write_mask/3"},{"type":"type","title":"OpcUA.Client.config_options/0","doc":"","ref":"OpcUA.Client.html#t:config_options/0"},{"type":"type","title":"OpcUA.Client.conn_params/0","doc":"","ref":"OpcUA.Client.html#t:conn_params/0"},{"type":"type","title":"OpcUA.Client.monitored_items_options/0","doc":"","ref":"OpcUA.Client.html#t:monitored_items_options/0"},{"type":"module","title":"OpcUA.Common","doc":"This module covers common functions for Client & Server behavior.","ref":"OpcUA.Common.html"},{"type":"behaviour","title":"OpcUA.Server","doc":"OPC UA Server API module.\n\nThis module provides functions for configuration, add/delete/read/write nodes and discovery a OPC UA Server.\n\n`OpcUA.Server` is implemented as a `__using__` macro so that you can put it in any module,\nyou can initialize your Server manually (see `test/server_tests/write_event_test.exs`) or by overwriting\n`configuration/1` and `address_space/1` to autoset  the configuration and information model. It also helps you to\nhandle Server's \"write value\" events by overwriting `handle_write/2` callback.\n\nThe following example shows a module that takes its configuration from the environment (see `test/server_tests/terraform_test.exs`):\n\n```elixir\ndefmodule MyServer do\n  use OpcUA.Server\n  alias OpcUA.Server\n\n  # Use the `init` function to configure your server.\n  def init({parent_pid, 103} = _user_init_state, opc_ua_server_pid) do\n    Server.start(opc_ua_server_pid)\n    %{parent_pid: parent_pid}\n  end\n\n  def configuration(_user_init_state), do: Application.get_env(:opex62541, :configuration, [])\n  def address_space(_user_init_state), do: Application.get_env(:opex62541, :address_space, [])\n\n  def handle_write(write_event, %{parent_pid: parent_pid} = state) do\n    send(parent_pid, write_event)\n    state\n  end\nend\n```\n\nBecause it is small a GenServer, it accepts the same [options](https://hexdocs.pm/elixir/GenServer.html#module-how-to-supervise) for supervision\nto configure the child spec and passes them along to `GenServer`:\n\n```elixir\ndefmodule MyModule do\n  use OpcUA.Server, restart: :transient, shutdown: 10_000\nend\n```","ref":"OpcUA.Server.html"},{"type":"function","title":"OpcUA.Server.add_all_endpoints/1","doc":"Adds endpoints for all configured security policies in each mode.","ref":"OpcUA.Server.html#add_all_endpoints/1"},{"type":"function","title":"OpcUA.Server.add_all_policies/2","doc":"Adds all supported security policies and sets up certificate validation procedures (no endpoint).\nThe following must be filled:\n  * `:private_key` -> binary() or function().\n  * `:certificate` -> binary() or function().","ref":"OpcUA.Server.html#add_all_policies/2"},{"type":"function","title":"OpcUA.Server.add_basic128rsa15_policy/2","doc":"Adds the security policy ``SecurityPolicy#Basic128Rsa15`` to the server with certicate (no endpoint).\nThe following must be filled:\n  * `:private_key` -> binary() or function().\n  * `:certificate` -> binary() or function().","ref":"OpcUA.Server.html#add_basic128rsa15_policy/2"},{"type":"function","title":"OpcUA.Server.add_basic256_policy/2","doc":"Adds the security policy ``SecurityPolicy#Basic256`` to the server with certicate (no endpoint).\nThe following must be filled:\n  * `:private_key` -> binary() or function().\n  * `:certificate` -> binary() or function().","ref":"OpcUA.Server.html#add_basic256_policy/2"},{"type":"function","title":"OpcUA.Server.add_basic256sha256_policy/2","doc":"Adds the security policy ``SecurityPolicy#Basic256Sha256`` to the server with certicate (no endpoint).\nThe following must be filled:\n  * `:private_key` -> binary() or function().\n  * `:certificate` -> binary() or function().","ref":"OpcUA.Server.html#add_basic256sha256_policy/2"},{"type":"function","title":"OpcUA.Server.add_data_type_node/2","doc":"Add a new data type node to the server.\nThe following must be filled:\n  * `:requested_new_node_id` -> %NodeID{}.\n  * `:parent_node_id` -> %NodeID{}.\n  * `:reference_type_node_id` -> %NodeID{}.\n  * `:browse_name` -> %QualifiedName{}.","ref":"OpcUA.Server.html#add_data_type_node/2"},{"type":"function","title":"OpcUA.Server.add_monitored_item/2","doc":"Create a local MonitoredItem with a sampling interval that detects data changes.\nThe following must be filled:\n  * `:monitored_item` -> %NodeID{}.\n  * `:sampling_time` -> double().","ref":"OpcUA.Server.html#add_monitored_item/2"},{"type":"function","title":"OpcUA.Server.add_namespace/2","doc":"Add a new namespace.","ref":"OpcUA.Server.html#add_namespace/2"},{"type":"function","title":"OpcUA.Server.add_none_policy/2","doc":"Adds the security policy ``SecurityPolicy#None`` to the server with certicate (no endpoint).\nThe following must be filled:\n  * `:certificate` -> binary() or function().","ref":"OpcUA.Server.html#add_none_policy/2"},{"type":"function","title":"OpcUA.Server.add_object_node/2","doc":"Add a new object node to the server.\nThe following must be filled:\n  * `:requested_new_node_id` -> %NodeID{}.\n  * `:parent_node_id` -> %NodeID{}.\n  * `:reference_type_node_id` -> %NodeID{}.\n  * `:browse_name` -> %QualifiedName{}.\n  * `:type_definition` -> %NodeID{}.","ref":"OpcUA.Server.html#add_object_node/2"},{"type":"function","title":"OpcUA.Server.add_object_type_node/2","doc":"Add a new object type node to the server.\nThe following must be filled:\n  * `:requested_new_node_id` -> %NodeID{}.\n  * `:parent_node_id` -> %NodeID{}.\n  * `:reference_type_node_id` -> %NodeID{}.\n  * `:browse_name` -> %QualifiedName{}.","ref":"OpcUA.Server.html#add_object_type_node/2"},{"type":"function","title":"OpcUA.Server.add_reference/2","doc":"Add a new reference in the server.\nThe following must be filled:\n  * `:source_id` -> %NodeID{}.\n  * `:reference_type_id` -> %NodeID{}.\n  * `:target_id` -> %NodeID{}.\n  * `:is_forward` -> boolean().","ref":"OpcUA.Server.html#add_reference/2"},{"type":"function","title":"OpcUA.Server.add_reference_type_node/2","doc":"Add a new reference type node to the server.\nThe following must be filled:\n  * `:requested_new_node_id` -> %NodeID{}.\n  * `:parent_node_id` -> %NodeID{}.\n  * `:reference_type_node_id` -> %NodeID{}.\n  * `:browse_name` -> %QualifiedName{}.","ref":"OpcUA.Server.html#add_reference_type_node/2"},{"type":"function","title":"OpcUA.Server.add_variable_node/2","doc":"Add a new variable node to the server.\nThe following must be filled:\n  * `:requested_new_node_id` -> %NodeID{}.\n  * `:parent_node_id` -> %NodeID{}.\n  * `:reference_type_node_id` -> %NodeID{}.\n  * `:browse_name` -> %QualifiedName{}.\n  * `:type_definition` -> %NodeID{}.","ref":"OpcUA.Server.html#add_variable_node/2"},{"type":"function","title":"OpcUA.Server.add_variable_type_node/2","doc":"Add a new variable type node to the server.\nThe following must be filled:\n  * `:requested_new_node_id` -> %NodeID{}.\n  * `:parent_node_id` -> %NodeID{}.\n  * `:reference_type_node_id` -> %NodeID{}.\n  * `:browse_name` -> %QualifiedName{}.\n  * `:type_definition` -> %NodeID{}.","ref":"OpcUA.Server.html#add_variable_type_node/2"},{"type":"function","title":"OpcUA.Server.add_view_node/2","doc":"Add a new view node to the server.\nThe following must be filled:\n  * `:requested_new_node_id` -> %NodeID{}.\n  * `:parent_node_id` -> %NodeID{}.\n  * `:reference_type_node_id` -> %NodeID{}.\n  * `:browse_name` -> %QualifiedName{}.","ref":"OpcUA.Server.html#add_view_node/2"},{"type":"callback","title":"OpcUA.Server.address_space/1","doc":"Optional callback that gets a list of nodes (with their attributes) to be automatically set.","ref":"OpcUA.Server.html#c:address_space/1"},{"type":"function","title":"OpcUA.Server.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"OpcUA.Server.html#child_spec/1"},{"type":"callback","title":"OpcUA.Server.configuration/1","doc":"Optional callback that gets the Server configuration and discovery connection parameters.","ref":"OpcUA.Server.html#c:configuration/1"},{"type":"function","title":"OpcUA.Server.delete_monitored_item/2","doc":"Deletes a local MonitoredItem.","ref":"OpcUA.Server.html#delete_monitored_item/2"},{"type":"function","title":"OpcUA.Server.delete_node/2","doc":"Deletes a node in the server.\nThe following must be filled:\n  * `:node_id` -> %NodeID{}.\n  * `:delete_references` -> boolean().","ref":"OpcUA.Server.html#delete_node/2"},{"type":"function","title":"OpcUA.Server.delete_reference/2","doc":"Deletes a reference in the server.\nThe following must be filled:\n  * `:source_id` -> %NodeID{}.\n  * `:reference_type_id` -> %NodeID{}.\n  * `:target_id` -> %NodeID{}.\n  * `:is_forward` -> boolean().\n  * `:delete_bidirectional` -> boolean().","ref":"OpcUA.Server.html#delete_reference/2"},{"type":"function","title":"OpcUA.Server.discovery_register/2","doc":"Registers a server in a discovery server.\nNOTE: The Server sends the request once started. Use port = 0 to dynamically port allocation.\n\nThe following must be filled:\n  * `:application_uri` -> binary().\n  * `:server_name` -> binary().\n  * `:endpoint` -> binary().\n  * `:timeout` -> boolean().","ref":"OpcUA.Server.html#discovery_register/2"},{"type":"function","title":"OpcUA.Server.discovery_unregister/1","doc":"Unregister the server from the discovery server.\nNOTE: Server must be started.","ref":"OpcUA.Server.html#discovery_unregister/1"},{"type":"function","title":"OpcUA.Server.get_config/1","doc":"Reads an internal Server Config.","ref":"OpcUA.Server.html#get_config/1"},{"type":"callback","title":"OpcUA.Server.handle_write/2","doc":"Optional callback that handles node values updates from a Client to a Server.\n\nIt's first argument will a tuple, in which its first element is the `node_id` of the updated node\nand the second element is the updated value.\n\nthe second argument it's the GenServer state (Parent process).","ref":"OpcUA.Server.html#c:handle_write/2"},{"type":"function","title":"OpcUA.Server.init/1","doc":"","ref":"OpcUA.Server.html#init/1"},{"type":"function","title":"OpcUA.Server.read_node_access_level/2","doc":"Reads 'Access level' of a node in the server.","ref":"OpcUA.Server.html#read_node_access_level/2"},{"type":"function","title":"OpcUA.Server.read_node_array_dimensions/2","doc":"Reads 'array_dimensions' of a node in the server.","ref":"OpcUA.Server.html#read_node_array_dimensions/2"},{"type":"function","title":"OpcUA.Server.read_node_browse_name/2","doc":"Reads the browse name attribute of a node in the server.","ref":"OpcUA.Server.html#read_node_browse_name/2"},{"type":"function","title":"OpcUA.Server.read_node_contains_no_loops/2","doc":"Reads 'contains_no_loops' attribute of a node in the server.","ref":"OpcUA.Server.html#read_node_contains_no_loops/2"},{"type":"function","title":"OpcUA.Server.read_node_data_type/2","doc":"Reads 'data_type' attribute of a node in the server.","ref":"OpcUA.Server.html#read_node_data_type/2"},{"type":"function","title":"OpcUA.Server.read_node_description/2","doc":"Reads description attribute of a node in the server.","ref":"OpcUA.Server.html#read_node_description/2"},{"type":"function","title":"OpcUA.Server.read_node_display_name/2","doc":"Reads the display name attribute of a node in the server.","ref":"OpcUA.Server.html#read_node_display_name/2"},{"type":"function","title":"OpcUA.Server.read_node_event_notifier/2","doc":"Reads 'event_notifier' attribute of a node in the server.","ref":"OpcUA.Server.html#read_node_event_notifier/2"},{"type":"function","title":"OpcUA.Server.read_node_executable/2","doc":"Reads 'Executable' attribute of a node in the server.","ref":"OpcUA.Server.html#read_node_executable/2"},{"type":"function","title":"OpcUA.Server.read_node_historizing/2","doc":"Reads 'Historizing' attribute of a node in the server.","ref":"OpcUA.Server.html#read_node_historizing/2"},{"type":"function","title":"OpcUA.Server.read_node_inverse_name/2","doc":"Reads 'Inverse name' attribute of a node in the server.","ref":"OpcUA.Server.html#read_node_inverse_name/2"},{"type":"function","title":"OpcUA.Server.read_node_is_abstract/2","doc":"Reads 'Is Abstract' attribute of a node in the server.","ref":"OpcUA.Server.html#read_node_is_abstract/2"},{"type":"function","title":"OpcUA.Server.read_node_minimum_sampling_interval/2","doc":"Reads 'Minimum Sampling Interval level' of a node in the server.","ref":"OpcUA.Server.html#read_node_minimum_sampling_interval/2"},{"type":"function","title":"OpcUA.Server.read_node_node_class/2","doc":"Reads the node_class attribute of a node in the server.","ref":"OpcUA.Server.html#read_node_node_class/2"},{"type":"function","title":"OpcUA.Server.read_node_node_id/2","doc":"Reads the node_id attribute of a node in the server.","ref":"OpcUA.Server.html#read_node_node_id/2"},{"type":"function","title":"OpcUA.Server.read_node_symmetric/2","doc":"Reads 'Symmetric' attribute of a node in the server.","ref":"OpcUA.Server.html#read_node_symmetric/2"},{"type":"function","title":"OpcUA.Server.read_node_value/3","doc":"Reads 'value' attribute of a node in the server.\nNote: If the value is an array you can search a scalar using `index` parameter.","ref":"OpcUA.Server.html#read_node_value/3"},{"type":"function","title":"OpcUA.Server.read_node_value_by_data_type/3","doc":"Reads 'Value' attribute (matching data type) of a node in the server.","ref":"OpcUA.Server.html#read_node_value_by_data_type/3"},{"type":"function","title":"OpcUA.Server.read_node_value_by_index/3","doc":"Reads 'value' attribute of a node in the server.\nNote: If the value is an array you can search a scalar using `index` parameter.","ref":"OpcUA.Server.html#read_node_value_by_index/3"},{"type":"function","title":"OpcUA.Server.read_node_value_rank/2","doc":"Reads 'Value Rank' of a node in the server.","ref":"OpcUA.Server.html#read_node_value_rank/2"},{"type":"function","title":"OpcUA.Server.read_node_write_mask/2","doc":"Reads 'Write Mask' attribute of a node in the server.","ref":"OpcUA.Server.html#read_node_write_mask/2"},{"type":"function","title":"OpcUA.Server.set_basics/1","doc":"Sets a default Server Config with no network layer and no endpoints.","ref":"OpcUA.Server.html#set_basics/1"},{"type":"function","title":"OpcUA.Server.set_default_config/1","doc":"Sets a default Server Config.","ref":"OpcUA.Server.html#set_default_config/1"},{"type":"function","title":"OpcUA.Server.set_default_config_with_certs/2","doc":"Creates a server configuration with all security policies for the given certificates.\nThe following must be filled:\n  * `:private_key` -> binary() or function().\n  * `:certificate` -> binary() or function().\n  * `:port` -> interger().","ref":"OpcUA.Server.html#set_default_config_with_certs/2"},{"type":"function","title":"OpcUA.Server.set_hostname/2","doc":"Sets the host name for the Server.","ref":"OpcUA.Server.html#set_hostname/2"},{"type":"function","title":"OpcUA.Server.set_lds_config/3","doc":"Sets the configuration for the a Server representing a local discovery server as a central instance.\nAny other server can register with this server using \"discovery_register\" function\nNOTE: before calling this function, this server should have the default configuration.\nLDS Servers only supports the Discovery Services. Cannot be used in combination with any other capability.\n\nThe following args must be filled:\n  * `:application_uri` -> binary().\n  * `:timeout` -> boolean().","ref":"OpcUA.Server.html#set_lds_config/3"},{"type":"function","title":"OpcUA.Server.set_network_tcp_layer/2","doc":"Sets a port number for the Server.","ref":"OpcUA.Server.html#set_network_tcp_layer/2"},{"type":"function","title":"OpcUA.Server.set_port/2","doc":"Sets a port number for the Server.","ref":"OpcUA.Server.html#set_port/2"},{"type":"function","title":"OpcUA.Server.set_users/2","doc":"Adds users (and passwords) the Server.\nUsers must be a tuple list ([{user, password}]).","ref":"OpcUA.Server.html#set_users/2"},{"type":"function","title":"OpcUA.Server.start/1","doc":"Start OPC UA Server.","ref":"OpcUA.Server.html#start/1"},{"type":"function","title":"OpcUA.Server.start_link/2","doc":"Starts up a OPC UA Server GenServer.","ref":"OpcUA.Server.html#start_link/2"},{"type":"function","title":"OpcUA.Server.stop/1","doc":"Stops a OPC UA Server GenServer.","ref":"OpcUA.Server.html#stop/1"},{"type":"function","title":"OpcUA.Server.stop_server/1","doc":"Stop OPC UA Server.","ref":"OpcUA.Server.html#stop_server/1"},{"type":"function","title":"OpcUA.Server.write_node_access_level/3","doc":"Change 'Access level' of a node in the server.","ref":"OpcUA.Server.html#write_node_access_level/3"},{"type":"function","title":"OpcUA.Server.write_node_array_dimensions/3","doc":"Change 'Array Dimensions' of a node in the server.","ref":"OpcUA.Server.html#write_node_array_dimensions/3"},{"type":"function","title":"OpcUA.Server.write_node_blank_array/4","doc":"Creates a blank 'value array' attribute of a node in the server.\nNote: the array must match with 'value_rank' and 'array_dimensions' attribute.","ref":"OpcUA.Server.html#write_node_blank_array/4"},{"type":"function","title":"OpcUA.Server.write_node_browse_name/3","doc":"Change the browse name of a node in the server.","ref":"OpcUA.Server.html#write_node_browse_name/3"},{"type":"function","title":"OpcUA.Server.write_node_data_type/3","doc":"Change 'data_type' attribute of a node in the server.","ref":"OpcUA.Server.html#write_node_data_type/3"},{"type":"function","title":"OpcUA.Server.write_node_description/4","doc":"Change description attribute of a node in the server.","ref":"OpcUA.Server.html#write_node_description/4"},{"type":"function","title":"OpcUA.Server.write_node_display_name/4","doc":"Change the display name attribute of a node in the server.","ref":"OpcUA.Server.html#write_node_display_name/4"},{"type":"function","title":"OpcUA.Server.write_node_event_notifier/3","doc":"Change 'event_notifier' attribute of a node in the server.","ref":"OpcUA.Server.html#write_node_event_notifier/3"},{"type":"function","title":"OpcUA.Server.write_node_executable/3","doc":"Change 'Executable' attribute of a node in the server.","ref":"OpcUA.Server.html#write_node_executable/3"},{"type":"function","title":"OpcUA.Server.write_node_historizing/3","doc":"Change 'Historizing' attribute of a node in the server.","ref":"OpcUA.Server.html#write_node_historizing/3"},{"type":"function","title":"OpcUA.Server.write_node_inverse_name/4","doc":"Change 'Inverse name' attribute of a node in the server.","ref":"OpcUA.Server.html#write_node_inverse_name/4"},{"type":"function","title":"OpcUA.Server.write_node_is_abstract/3","doc":"Change 'Is Abstract' attribute of a node in the server.","ref":"OpcUA.Server.html#write_node_is_abstract/3"},{"type":"function","title":"OpcUA.Server.write_node_minimum_sampling_interval/3","doc":"Change 'Minimum Sampling Interval level' of a node in the server.","ref":"OpcUA.Server.html#write_node_minimum_sampling_interval/3"},{"type":"function","title":"OpcUA.Server.write_node_value/5","doc":"Change 'Value' attribute of a node in the server.","ref":"OpcUA.Server.html#write_node_value/5"},{"type":"function","title":"OpcUA.Server.write_node_value_rank/3","doc":"Change 'Value rank' of a node in the server.\n\nThis attribute indicates whether the value attribute of the variable is an array and how many dimensions the array has.\nIt may have the following values:\n\nvalue_rank >= 1: the value is an array with the specified number of dimensions\nvalue_rank =  0: the value is an array with one or more dimensions\nvalue_rank = -1: the value is a scalar\nvalue_rank = -2: the value can be a scalar or an array with any number of dimensions\nvalue_rank = -3: the value can be a scalar or a one dimensional array","ref":"OpcUA.Server.html#write_node_value_rank/3"},{"type":"function","title":"OpcUA.Server.write_node_write_mask/3","doc":"Change 'Write Mask' attribute of a node in the server.","ref":"OpcUA.Server.html#write_node_write_mask/3"},{"type":"type","title":"OpcUA.Server.address_space_list/0","doc":"","ref":"OpcUA.Server.html#t:address_space_list/0"},{"type":"type","title":"OpcUA.Server.config_options/0","doc":"","ref":"OpcUA.Server.html#t:config_options/0"},{"type":"type","title":"OpcUA.Server.config_params/0","doc":"","ref":"OpcUA.Server.html#t:config_params/0"},{"type":"module","title":"OpcUA.BaseNodeAttrs","doc":"Base Node Attributes\n\n  Nodes contain attributes according to their node type. The base node\n  attributes are common to all node types. In the OPC UA `services`,\n  attributes are referred to via the `nodeid` of the containing node and\n  an integer `attribute-id`.\n\n  In opex62541 we set `node_id` during the node definition.\n\n  TODO: add node_id, node_class, references_size, :reference backend","ref":"OpcUA.BaseNodeAttrs.html"},{"type":"module","title":"OpcUA.DataTypeNode","doc":"DataTypeNode\n\n  DataTypes represent simple and structured data types. DataTypes may contain\n  arrays. But they always describe the structure of a single instance. In\n  open62541, DataTypeNodes in the information model hierarchy are matched to\n  ``UA_DataType`` type descriptions for :ref:`generic-types` via their NodeId.\n\n  Abstract DataTypes (e.g. ``Number``) cannot be the type of actual values.\n  They are used to constrain values to possible child DataTypes (e.g.\n  ``UInt32``).","ref":"OpcUA.DataTypeNode.html"},{"type":"function","title":"OpcUA.DataTypeNode.new/2","doc":"","ref":"OpcUA.DataTypeNode.html#new/2"},{"type":"module","title":"OpcUA.MethodNode","doc":"MethodNode\n\n  Methods define callable functions and are invoked using the :ref:`Call\n   ` service. MethodNodes may have special properties (variable\n  childen with a ``hasProperty`` reference) with the :ref:`qualifiedname` ``(0,\n  \"InputArguments\")`` and ``(0, \"OutputArguments\")``. The input and output\n  arguments are both described via an array of ``UA_Argument``. While the Call\n  service uses a generic array of :ref:`variant` for input and output, the\n  actual argument values are checked to match the signature of the MethodNode.\n\n  Note that the same MethodNode may be referenced from several objects (and\n  object types). For this, the NodeId of the method *and of the object\n  providing context* is part of a Call request message.","ref":"OpcUA.MethodNode.html"},{"type":"function","title":"OpcUA.MethodNode.new/2","doc":"","ref":"OpcUA.MethodNode.html#new/2"},{"type":"module","title":"OpcUA.MonitoredItem","doc":"A Monitored Item is used to request a server for notifications of each change of value in a specific node.","ref":"OpcUA.MonitoredItem.html"},{"type":"function","title":"OpcUA.MonitoredItem.new/1","doc":"Creates a structure for an Monitored Item of an existing node in a Server.\nThe following options must be filled:\n  * `:monitored_item` -> %NodeId().\n  * `:sampling_time` -> double().\n  * `:subscription_id` -> integer().","ref":"OpcUA.MonitoredItem.html#new/1"},{"type":"module","title":"OpcUA.ObjectNode","doc":"ObjectNode\n\n  Objects are used to represent systems, system components, real-world objects\n  and software objects. Objects are instances of an `object\n  type ` and may contain variables, methods and further\n  objects.","ref":"OpcUA.ObjectNode.html"},{"type":"function","title":"OpcUA.ObjectNode.new/2","doc":"","ref":"OpcUA.ObjectNode.html#new/2"},{"type":"module","title":"OpcUA.ObjectTypeNode","doc":"ObjectTypeNode\n\n  ObjectTypes provide definitions for Objects. Abstract objects cannot be\n  instantiated.","ref":"OpcUA.ObjectTypeNode.html"},{"type":"function","title":"OpcUA.ObjectTypeNode.new/2","doc":"","ref":"OpcUA.ObjectTypeNode.html#new/2"},{"type":"module","title":"OpcUA.ReferenceNode","doc":"","ref":"OpcUA.ReferenceNode.html"},{"type":"function","title":"OpcUA.ReferenceNode.new/2","doc":"","ref":"OpcUA.ReferenceNode.html#new/2"},{"type":"module","title":"OpcUA.ReferenceTypeNode","doc":"ReferenceTypeNode\n\n  Each reference between two nodes is typed with a ReferenceType that gives\n  meaning to the relation. The OPC UA standard defines a set of ReferenceTypes\n  as a mandatory part of OPC UA information models.\n\n  - Abstract ReferenceTypes cannot be used in actual references and are only\n    used to structure the ReferenceTypes hierarchy\n  - Symmetric references have the same meaning from the perspective of the\n    source and target node","ref":"OpcUA.ReferenceTypeNode.html"},{"type":"function","title":"OpcUA.ReferenceTypeNode.new/2","doc":"","ref":"OpcUA.ReferenceTypeNode.html#new/2"},{"type":"module","title":"OpcUA.VariableNode","doc":"VariableNode\n\n  Variables store values in a `value` together with metadata for introspection.\n  Most notably, the attributes data type, `value_rank` and array dimensions constrain the possible values the variable can take\n  on.\n\n  Variables come in two flavours: properties and datavariables. Properties are\n  related to a parent with a ``hasProperty`` reference and may not have child\n  nodes themselves. Datavariables may contain properties (``hasProperty``) and\n  also datavariables (``hasComponents``).\n\n  All variables are instances of some `variabletypenode` in return\n  constraining the possible data type, value rank and array dimensions\n  attributes.\n\n  Data Type\n\n  The (scalar) data type of the variable is constrained to be of a specific\n  type or one of its children in the type hierarchy. The data type is given as\n  a NodeId pointing to a `DataTypeNode` in the type hierarchy. See the\n  Section `DataTypeNode` for more details.\n\n  If the data type attribute points to ``UInt32``, then the value attribute\n  must be of that exact type since ``UInt32`` does not have children in the\n  type hierarchy. If the data type attribute points ``Number``, then the type\n  of the value attribute may still be ``UInt32``, but also ``Float`` or\n  ``Byte``.\n\n  Consistency between the data type attribute in the variable and its\n  `VariableTypeNode` is ensured.\n\n  Value Rank\n\n\n  This attribute indicates whether the value attribute of the variable is an\n  array and how many dimensions the array has. It may have the following\n  values:\n\n  - ``n >= 1``: the value is an array with the specified number of dimensions\n  - ``n =  0``: the value is an array with one or more dimensions\n  - ``n = -1``: the value is a scalar\n  - ``n = -2``: the value can be a scalar or an array with any number of dimensions\n  - ``n = -3``: the value can be a scalar or a one dimensional array\n\n  Consistency between the value rank attribute in the variable and its\n  `variabletypenode` is ensured.\n\n  TODO:\n  Array Dimensions\n\n  If the value rank permits the value to be a (multi-dimensional) array, the\n  exact length in each dimensions can be further constrained with this\n  attribute.\n\n  - For positive lengths, the variable value is guaranteed to be of the same\n    length in this dimension.\n  - The dimension length zero is a wildcard and the actual value may have any\n    length in this dimension.\n\n  Consistency between the array dimensions attribute in the variable and its\n  `variabletypenode` is ensured.\n\n  Indicates whether a variable contains data inline or whether it points to an\n  external data source.","ref":"OpcUA.VariableNode.html"},{"type":"function","title":"OpcUA.VariableNode.new/2","doc":"","ref":"OpcUA.VariableNode.html#new/2"},{"type":"module","title":"OpcUA.VariableTypeNode","doc":"VariableTypeNode\n\n  VariableTypes are used to provide type definitions for variables.\n  VariableTypes constrain the data type, value rank and array dimensions\n  attributes of variable instances. Furthermore, instantiating from a specific\n  variable type may provide semantic information. For example, an instance from\n  `MotorTemperatureVariableType` is more meaningful than a float variable\n  instantiated from `BaseDataVariable`.","ref":"OpcUA.VariableTypeNode.html"},{"type":"function","title":"OpcUA.VariableTypeNode.new/2","doc":"","ref":"OpcUA.VariableTypeNode.html#new/2"},{"type":"module","title":"OpcUA.ViewNode","doc":"ViewNode\n\n  Each View defines a subset of the Nodes in the AddressSpace. Views can be\n  used when browsing an information model to focus on a subset of nodes and\n  references only. ViewNodes can be created and be interacted with. But their\n  use in the :ref:`Browse ` service is currently unsupported in\n  open62541.","ref":"OpcUA.ViewNode.html"},{"type":"function","title":"OpcUA.ViewNode.new/2","doc":"","ref":"OpcUA.ViewNode.html#new/2"},{"type":"module","title":"OpcUA.ExpandedNodeId","doc":"A NodeId that allows the namespace URI to be specified instead of an index.","ref":"OpcUA.ExpandedNodeId.html"},{"type":"function","title":"OpcUA.ExpandedNodeId.new/1","doc":"Creates a structure for an expanded node in the address space of an OPC UA Server.","ref":"OpcUA.ExpandedNodeId.html#new/1"},{"type":"module","title":"OpcUA.NodeId","doc":"An identifier for a node in the address space of an OPC UA Server.\n\nAn OPC UA information model is made up of nodes and references between nodes.\nEvery node has a unique NodeId. NodeIds refer to a `namespace` with an additional\n`identifier` value that can be an integer, a string, a guid or a bytestring depending on the selected\n`identifier_type`.","ref":"OpcUA.NodeId.html"},{"type":"function","title":"OpcUA.NodeId.new/1","doc":"Creates a structure for a node in the address space of an OPC UA Server.","ref":"OpcUA.NodeId.html#new/1"},{"type":"module","title":"OpcUA.QualifiedName","doc":"A name qualified by a namespace.","ref":"OpcUA.QualifiedName.html"},{"type":"function","title":"OpcUA.QualifiedName.new/1","doc":"Creates a structure for a name qualified by a namespace.","ref":"OpcUA.QualifiedName.html#new/1"},{"type":"module","title":"Opex62541","doc":"Documentation for Opex62541.","ref":"Opex62541.html"},{"type":"extras","title":"README","doc":"***\n \n \n   \n \n \n\nOpex62541 is an Elixir wrapper for the [open62541](https://github.com/open62541/open62541) library; open62541 is an open-source implementation of OPC UA (OPC Unified Architecture) aka IEC 62541 licensed under Mozilla Public License v2.0.","ref":"readme.html"},{"type":"extras","title":"Content - README","doc":"- [Features](#features)\n- [Installation](#installation)\n  - [Compatibility](#compatibility)\n  - [Nerves](#nerves)\n  - [Customized build](#customized-build)\n- [Documentation](#documentation)\n- [Contributing](#contributing)\n- [License](#license)\n- [TODO](#TODO)","ref":"readme.html#content"},{"type":"extras","title":"Features - README","doc":"This library implements the following features from [open62541](https://github.com/open62541/open62541):\n- Communication Stack\n  - OPC UA binary protocol\n  - Secure communication with encrypted messages\n- Server\n  - Support for all OPC UA node types\n  - Access control for individual nodes\n  - Support for adding and removing nodes and references also at runtime.\n  - Support for inheritance and instantiation of object and variable-types.\n  - Support for subscriptions/monitored items.\n- Client\n  - All OPC UA services supported\n  - Support for subscriptions/monitored items.","ref":"readme.html#features"},{"type":"extras","title":"Installation - README","doc":"To install this package, add `opex62541` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:opex62541, git: \"https://github.com/valiot/opex62541\"}\n  ]\nend\n```\n\n#","ref":"readme.html#installation"},{"type":"extras","title":"Compatibility - README","doc":"Opex62541 was developed for open62541 \"1.0.0\" with Elixir 1.10.0. It was tested on:\n  * Ubuntu 18.04, 20.04\n  * Raspbian OS (Raspberry Pi 3B+)\n  * Nerves (Raspberry Pi 3B+)\n\nAdding the following `opex62541` dependencies to build the package:\n\n```bash\nsudo apt-get install git build-essential gcc pkg-config cmake python libmbedtls-dev\n```\n\n#","ref":"readme.html#compatibility"},{"type":"extras","title":"Nerves - README","doc":"[Nerves](https://www.nerves-project.org) is a complete IoT platform and infrastructure to build and deploy maintainable embedded systems to boards such as Raspberry Pi or Beaglebone.\n\nTo add `opex62541` dependency (`mbedtls`) to your Nerves project, you need to create a [Nerves Custom System](https://hexdocs.pm/nerves/customizing-systems.html#content) and add the following lines to the `nerves_defconfig` file:\n\n```bash\nBR2_PACKAGE_MBEDTLS=y\nBR2_PACKAGE_MBEDTLS_COMPRESSION=y\n```\n\n#","ref":"readme.html#nerves"},{"type":"extras","title":"Customized builds - README","doc":"By default, Opex62541 downloads and compiles the `v1.0` release of open62541. If you want to compile it manually or change the default version, use the following example commands to set the desired env variables:\n\n```bash\nexport MANUAL_BUILD=true\n\nexport OPEN62541_DOWNLOAD_VERSION=v1.0.1\n```\nThe open62541 project uses CMake to manage the build options for code generation and to generate build projects for the different systems and IDEs. To overwrite the default options, use `OPEN62541_BUILD_ARGS` as follows:\n\n```bash\nexport OPEN62541_BUILD_ARGS='-DCMAKE_BUILD_TYPE=Release -DUA_NAMESPACE_ZERO=MINIMAL'\n```\n\nDefault values for `OPEN62541_BUILD_ARGS` are `-DBUILD_SHARED_LIBS=ON -DCMAKE_BUILD_TYPE=RelWithDebInfo -DUA_NAMESPACE_ZERO=FULL -DUA_LOGLEVEL=601 -DUA_ENABLE_DISCOVERY_MULTICAST=ON -DUA_ENABLE_AMALGAMATION=ON -DUA_ENABLE_ENCRYPTION=ON`.","ref":"readme.html#customized-builds"},{"type":"extras","title":"Docker Container - README","doc":"To build the container locally use:\n\n```bash\ngit clone https://github.com/valiot/opex62541\ncd opex62541\ndocker build -t   .\n```\n\nYou can use this container to test this application.","ref":"readme.html#docker-container"},{"type":"extras","title":"Documentation - README","doc":"For detailed documentation and tutorials refer to [hexdocs.pm](https://hexdocs.pm/opex62541).","ref":"readme.html#documentation"},{"type":"extras","title":"Contributing - README","doc":"* Fork our repository on Github.\n  * Fix or add what is needed.\n  * Commit to your repository\n  * Issue a Github Pull Request.\n  * Fill the pull request template.\n\nIf you wish to clone this repository, use:\n```\ngit clone https://github.com/valiot/opex62541.git\n```","ref":"readme.html#contributing"},{"type":"extras","title":"License - README","doc":"See [LICENSE](https://github.com/valiot/opex62541/blob/master/LICENSE).","ref":"readme.html#license"},{"type":"extras","title":"TODO - README","doc":"* **Methods**\n  * **OPC UA PubSub**\n  * **Better C code handling for the Client and Server common code**","ref":"readme.html#todo"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"0.1.4 - Changelog","doc":"* Elixir 1.15 (OTP 26) warnings fixed.\n* Dependencies update.","ref":"changelog.html#0-1-4"},{"type":"extras","title":"0.1.3 - Changelog","doc":"* Logos added.\n* Dependencies update.","ref":"changelog.html#0-1-3"},{"type":"extras","title":"0.1.2 - Changelog","doc":"* Elixir 1.12 (OTP 24) support.\n* Dockerfile added.\n* `README.md` update.","ref":"changelog.html#0-1-2"},{"type":"extras","title":"0.1.1 - Changelog","doc":"* `exdoc` dependency added.\n* `Hexdocs` support.\n* `README.md` update.\n* Guide files & section added.\n* Changelog file added.","ref":"changelog.html#0-1-1"},{"type":"extras","title":"Introduction","doc":"# Introduction","ref":"introduction.html"},{"type":"extras","title":"Content - Introduction","doc":"- [OPC UA](#opc-ua)\n- [Tutorials](#tutorials)\n  - [Lifecycle](#lifecycle)\n  - [Security](#security)\n  - [Discovery](#discovery)\n  - [Information Manipulation](#information-manipulation)\n  - [Elixir Integration](#elixir-integration)\n- [Development](#development)","ref":"introduction.html#content"},{"type":"extras","title":"OPC UA - Introduction","doc":"OPC UA (Unified Architecture) extends the great success of the OPC communication protocol for data acquisition, information modeling (object-oriented), and communication between plant and applications (client-server) reliably and securely. OPC UA is based on a service-oriented approach defined by the IEC 62451 standard.\n\nOpex62541 wraps the basic functions of OPC UA client and server.","ref":"introduction.html#opc-ua"},{"type":"extras","title":"Tutorials - Introduction","doc":"The purpose of the following sections is to provide a guided tour of using this library in simple applications.\n\nFirst, we will explain the most important features and the manual way to use the library (such as configuration, connectivity, data modeling, and execution).\n\nAnd finally, we will show automatic methods to configure and integrate the library with the elixir module.\n\n#","ref":"introduction.html#tutorials"},{"type":"extras","title":"Lifecycle - Introduction","doc":"This section shows how the life cycle of an OPC UA client and server should be executed. We will see configuration and connectivity topics.\n\n#","ref":"introduction.html#lifecycle"},{"type":"extras","title":"Security - Introduction","doc":"Next, we will focus on adding a more robust security layer. We will show you the security modes we support and how to configure them.\n\n#","ref":"introduction.html#security"},{"type":"extras","title":"Discovery - Introduction","doc":"We will discuss the interaction between an OPC UA client or server and a network, i.e., on the server-side, how to become visible to clients, and on the client-side, how to scan the network to identify a server. In this section, we will configure a Local Discovery Server (LDS), register a server in it, and retrieve information.\n\n#","ref":"introduction.html#discovery"},{"type":"extras","title":"Information Manipulation - Introduction","doc":"We will explore how the information is modeled in OPC UA, the methods to add the model to the server, and how clients can extract the data. Topics such as Address space, Nodes, Monitored Items, etc., will be covered.\n\n#","ref":"introduction.html#information-manipulation"},{"type":"extras","title":"Elixir Integration - Introduction","doc":"Finally, we will discuss a callback-based interface that facilitates the configuration and integration between the server or client and an elixir module.","ref":"introduction.html#elixir-integration"},{"type":"extras","title":"Development - Introduction","doc":"Opex62541 is an Elixir wrapper for the open62541 library (C Code). Therefore, two middlewares ([opc_ua_client.c](https://github.com/valiot/opex62541/blob/master/src/opc_ua_client.c) and [opc_ua_server.c](https://github.com/valiot/opex62541/blob/master/src/opc_ua_server.c)) were made to execute open62541 API functions based on stdio messages, this communication between Elixir and C middleware is established by creating a port ([Erlang's Port Module](http://erlang.org/doc/tutorial/c_port.html)), which allows possible errors in the C code to be contained.","ref":"introduction.html#development"},{"type":"extras","title":"Lifecycle","doc":"# Lifecycle\n\nThis tutorial will cover the following topics:","ref":"lifecycle.html"},{"type":"extras","title":"Content - Lifecycle","doc":"- [Server](#server)\n  - [Server Configuration](#server-configuration)\n  - [Basic Authentication](#basic-authentication)\n  - [Start / Stop](#start-/-stop)\n- [Client](#client)\n  - [Client Configuration](#client-configuration)\n  - [Connection](#connection)\n    - [By URL](#by-url)\n    - [By Username](#by-username)\n    - [No Session](#no-session)","ref":"lifecycle.html#content"},{"type":"extras","title":"Server - Lifecycle","doc":"The code in this example shows the three parts for OPC UA Server lifecycle management: Creating a Server, set configuration Server, and running the Server. Basic use of the [Server](https://hexdocs.pm/opex62541/OpcUA.Server.html) module.\n\nThe OPC UA Server is based on a `GenServer`. Therefore it can spawn a new server by using `start_link`.\n\n```elixir\nalias OpcUA.Server\n{:ok, server_pid} = Server.start_link()\n```\n\n#","ref":"lifecycle.html#server"},{"type":"extras","title":"Server Configuration - Lifecycle","doc":"The easiest way to configure the server is using the `set_default_config/1` function, as shown in the following code:\n\n```elixir\n:ok = Server.set_default_config(server_pid)\n```\n\nBy default the `hostname` is given by the localhost, however it is possible to overwrite its value by using  `set_hostname/2`:\n\n```elixir\n:ok = Server.set_hostname(server_pid, \"opex-server\")\n```\n**Note**: Be aware of DNS configuration.\n\nIt is also possible to change the default port as follows.\n\n```elixir\n:ok = Server.set_port(server_pid, 4040)\n```\n\n**Note**: Default port value is 4840.\n\n#","ref":"lifecycle.html#server-configuration"},{"type":"extras","title":"Basic Authentication - Lifecycle","doc":"Security is a fundamental aspect of OPC UA. It is possible to restrict access to the server by defining authorized users as shown bellow:\n\n```elixir\n:ok = Server.set_users(server_pid, [{\"alde103\", \"103alde\"}, {\"pedro\", \"ordep\"}])\n```\n\nIn this example, we declare two users (alde103 and pedro) with their respective passwords.\n\n**Note**: This is not the best way to secure your OPC UA Server. Refer to [Security](https://hexdocs.pm/opex62541/doc/security.html) tutorial for a better approach.\n\n#","ref":"lifecycle.html#basic-authentication"},{"type":"extras","title":"Start / Stop - Lifecycle","doc":"At this point, the server will not accept requests yet. Use the `start/1` function to start the server explicitly.\n\n```elixir\n:ok = Server.start(server_pid)\n```\n\nThe Server can be stopped with the `stop_server/1` function.\n\n```elixir\n:ok = Server.stop_server(server_pid)\n```","ref":"lifecycle.html#start-stop"},{"type":"extras","title":"Client - Lifecycle","doc":"The code in this example shows the three parts for OPC UA Client lifecycle management: creating, configuring, and connecting a client to a server. Basic use of the [Client](https://hexdocs.pm/opex62541/OpcUA.Client.html) module.\n\nThe OPC UA Client is based on a `GenServer`. Therefore it can spawn a new server by using `start_link`.\n\n```elixir\nalias OpcUA.Client\n{:ok, client_pid} = Client.start_link()\n```\n\n#","ref":"lifecycle.html#client"},{"type":"extras","title":"Configuration - Lifecycle","doc":"The client can be configured using the following function,\n\n```elixir\n:ok = Client.set_config(client_pid)\n```\nit is possible to overwrite some connection parameters (such as timeouts) by using  `set_hostname/2`,\n\n```elixir\nconfig = %{\n  \"requestedSessionTimeout\" => 12000,\n  \"secureChannelLifeTime\" => 6000,\n  \"timeout\" => 500\n}\n:ok = Client.set_config(client_pid, config)\n```\n\n**Note**: Use `get_config/1` function to check the current client configuration.\n\n#","ref":"lifecycle.html#configuration"},{"type":"extras","title":"Connection - Lifecycle","doc":"OPC UA provides a client-server communication model that includes status information. This status information is associated with a session.\n\nThe user can check the connection status through `get_state/1` function:\n\n```elixir\n{:ok,  \"Disconnected\"} = Client.get_state(client_pid)\n```\n\n##","ref":"lifecycle.html#connection"},{"type":"extras","title":"By URL - Lifecycle","doc":"The Client can connect to a Server by using `connect_by_url/2` and providing the Server's URL as follows:\n\n```elixir\nurl = \"opc.tcp://localhost:4048/\"\n:ok = Client.connect_by_url(client_pid, url: url)\n{:ok,  \"Session\"} = Client.get_state(client_pid)\n```\n\n##","ref":"lifecycle.html#by-url"},{"type":"extras","title":"By Username - Lifecycle","doc":"If the Server has some user restriction, the client must provide the username and password to access the Server. If this is your case, use `connect_by_username/2`\n\n```elixir\nurl = \"opc.tcp://localhost:4840/\"\nuser = \"alde103\"\npassword = \"103adle\"\n:ok = Client.connect_by_username(client_pid, url: url, user: user, password: password)\n```\n\n##","ref":"lifecycle.html#by-username"},{"type":"extras","title":"No Session - Lifecycle","doc":"There may be situations where a session is not required. For those cases use `connect_no_session/2`\n\n```elixir\nurl = \"opc.tcp://localhost:4048/\"\n:ok = Client.connect_no_session(c_pid, url: url)\n{:ok,  \"Secure Channel\"} = Client.get_state(c_pid)\n```","ref":"lifecycle.html#no-session"},{"type":"extras","title":"Security","doc":"# Security\n\nThis tutorial will cover the following topics:","ref":"security.html"},{"type":"extras","title":"Content - Security","doc":"- [Security Model](#security-model)\n- [Server](#server)\n  - [Manual Configuration](#manual-configuration)\n- [Client](#client)\n  - [Security Modes](#connection)","ref":"security.html#content"},{"type":"extras","title":"Security Model - Security","doc":"The OPC UA security model is implemented by defining a secure channel on which a session is based. A secure channel has the following features:\n* Digital signatures ensure the integrity of the data.\n* Encryption ensures confidentiality.\n* Authentication and authorization of applications by using X.509 certificates.\n\nA secure channel is related to an endpoint, and each server offers one or more endpoints.\nEach endpoint has the following features:\n* Server application instance certificate: This is the public key of the server used by the client to make the exchange of data secure.\n* Security policy: A set of algorithms used in security mechanisms.\n* Security mode: There are three different modes: None, Sign, or SignAndEncrypt.\n* Authentication: The mechanisms used to authenticate a user during the creation of a session (by username, a certificate, or anonymous authentication).\n* Transport protocol: Defines the network protocol that is going to be used.\n* Endpoint URL: The network address used by the client to establish a secure channel with the endpoint.\n\nSince this feature requires certificates to work, it is the user's responsibility to provide a valid certificate to the server or client.","ref":"security.html#security-model"},{"type":"extras","title":"Server - Security","doc":"Assuming that an OPC UA Server has been created as shown in [Lifecycle](https://hexdocs.pm/opex62541/doc/lifecycle.html) tutorial.\n\nThe easiest way to configure the security requirements of the server is by using the `set_default_config_with_certs/2` function, as shown in the following code:\n\n```elixir\nalias OpcUA.Server\n\ncerts_config = [\n  port: 4840,\n  certificate: File.read!(\"./test/demo_certs/server_cert.der\"),\n  private_key: File.read!(\"./test/demo_certs/server_key.der\")\n]\n\n:ok = Server.set_default_config_with_certs(server_pid, certs_config)\n```","ref":"security.html#server"},{"type":"extras","title":"Manual Configuration - Security","doc":"By default, the previous method sets all internal basic configuration, a TCP network layer (with port 4840), and all supported security policies. However, this configuration can be done manually to fit your application.\n\nNext we will detail step by step how to set the manual configuration as follows:\n\n**Step 1:**  Set the internal (open62451) basics:\n\n```elixir\n:ok = Server.set_basics(server_pid)\n```\n\n**Step 2:**  Set the Transport Protocol with a given port (4041, for example):\n\n```elixir\n:ok = Server.set_network_tcp_layer(server_pid, 4041)\n```\n\n**Step 3:**  Set the Security Policies, Currently this library supports the following:\n  * None.\n  * Basic128Rsa15.\n  * Basic256\n  * Basic256Sha256.\n\nThe following code adds the security policy ``SecurityPolicy#None`` to the server.\n\n```elixir\ncerts_info = [\n  certificate: File.read!(\"./test/demo_certs/server_cert.der\")\n]\n\n:ok = Server.add_none_policy(server_pid, certs_info)\n```\n\nTo add the security policy ``SecurityPolicy#Basic128Rsa15`` to the server use the next code,\n\n```elixir\ncerts_info = [\n  certificate: File.read!(\"./test/demo_certs/server_cert.der\"),\n  private_key: File.read!(\"./test/demo_certs/server_key.der\")\n]\n\n:ok = Server.add_basic128rsa15_policy(server_pid, certs_info)\n```\n\nthe next code adds the security policy ``SecurityPolicy#Basic256`` to the server,\n\n```elixir\ncerts_info = [\n  certificate: File.read!(\"./test/demo_certs/server_cert.der\"),\n  private_key: File.read!(\"./test/demo_certs/server_key.der\")\n]\n\n:ok = Server.add_basic256_policy(server_pid, certs_info)\n```\n\nthe security policy ``SecurityPolicy#Basic256Sha256`` can be added to the server using `add_basic256sha256_policy/2`,\n\n```elixir\ncerts_info = [\n  certificate: File.read!(\"./test/demo_certs/server_cert.der\"),\n  private_key: File.read!(\"./test/demo_certs/server_key.der\")\n]\n\n:ok = Server.add_basic256sha256_policy(server_pid, certs_info)\n```\n\nUsing `add_all_policies/2` adds all supported security policies and sets up certificate validation procedures.\n\n```elixir\ncerts_info = [\n  certificate: File.read!(\"./test/demo_certs/server_cert.der\"),\n  private_key: File.read!(\"./test/demo_certs/server_key.der\")\n]\n\n:ok = Server.add_all_policies(server_pid, certs_info)\n```\n\n**Step 4:** Finally, the following code adds an endpoint for every configured security policy to the server.\n\n```elixir\n:ok = Server.add_all_endpoints(server_pid, 4041)\n```","ref":"security.html#manual-configuration"},{"type":"extras","title":"Client - Security","doc":"Assuming that an OPC UA Client has been created as shown in [Lifecycle](https://hexdocs.pm/opex62541/doc/lifecycle.html) tutorial. The certificate and private key can be set as follow:\n\n```elixir\ncerts_config = [\n  security_mode: 2,\n  certificate: File.read!(\"./test/demo_certs/client_cert.der\"),\n  private_key: File.read!(\"./test/demo_certs/client_key.der\")\n]\n\n:ok = Client.set_config_with_certs(client_pid, certs_config)\n```","ref":"security.html#client"},{"type":"extras","title":"Security Modes - Security","doc":"If the certificate is reliable, the client sends an Open Secure Channel request in line with the security policy and the security mode of the selected session endpoint; in the `set_config_with_certs/2`, the `security_mode` option only accepts integers ([1, 2, 3]) and represents the following:\n* None (1), the request will be sent without any security mechanisms.\n* Sign (2), the request will be sent using the client's private key as a signature.\n* SignAndEncrypt (3), the request will be sent after encrypting it using the server's public key.\n\nOnce the client is configured, you can connect the client to the server using the functions exposed in the [Lifecycle](https://hexdocs.pm/opex62541/doc/lifecycle.html) tutorial.","ref":"security.html#security-modes"},{"type":"extras","title":"Discovery","doc":"# Discovery\n\nThis tutorial will cover the following topics:","ref":"discovery.html"},{"type":"extras","title":"Content - Discovery","doc":"- [Local Discovery Server](#local-discovery-server)\n- [Server](#server)\n  - [LDS Configuration](#lds-configuration)\n  - [Registration](#registration)\n- [Client](#client)\n  - [Scanning Network](#scan-network)","ref":"discovery.html#content"},{"type":"extras","title":"Local Discovery Server - Discovery","doc":"The Local Discovery Server (LDS) provides the necessary infrastructure to publicly expose the OPC UA Servers available on a given computer.\n\nOPC UA Servers will periodically connect to the LDS and Register themselves as being available. This periodic activity means that the list of available OPC UA servers is always current and means that an OPC UA client can immediately connect to any of them.","ref":"discovery.html#local-discovery-server"},{"type":"extras","title":"Server - Discovery","doc":"Assuming that an OPC UA Server has been created and configured as shown in [Lifecycle](https://hexdocs.pm/opex62541/doc/lifecycle.html) tutorial.","ref":"discovery.html#server"},{"type":"extras","title":"LDS Configuration - Discovery","doc":"To spawn an LDS server, follow these steps.\n\n```elixir\nalias OpcUA.Server\n\n{:ok, lds_pid} = Server.start_link()\n\n:ok = Server.set_default_config(lds_pid)\n\napplication_uri = \"urn:opex62541.test.local_discovery_server\"\n:ok = Server.set_lds_config(lds_pid, application_uri)\n```\n\n**Note**: LDS Servers only supports the Discovery Services. Therefore, it cannot be used in combination with any other capability.","ref":"discovery.html#lds-configuration"},{"type":"extras","title":"Registration - Discovery","doc":"The LDS maintains a list of available servers which servers may use to announce their existence to clients. Any other server can register with this server using `discovery_register/2` function as shown below:\n\n```elixir\nalias OpcUA.Server\n\n{:ok, server_pid} = Server.start_link()\n\n:ok = Server.set_default_config(server_pid)\n\napplication_uri = \"urn:opex62541.test.local_register_server\"\n:ok = Server.discovery_register(server_pid,\n        application_uri: application_uri,\n        server_name: \"TestRegister\",\n        endpoint: \"opc.tcp://localhost:4840\"\n      )\n```\nThe `endpoint` option represents the LDS endpoint, and the `server_name` is how the registered server will be visible in the network.\n\nThe `discovery_unregister` can be used to delete the desired server from the LDS server.\n\n```elixir\n:ok = Server.discovery_unregister(server_pid)\n```","ref":"discovery.html#registration"},{"type":"extras","title":"Client - Discovery","doc":"Clients may request a list of all available servers from the discovery server (LDS) and then use the GetEndpoints service to get the connection information from a server.\n\nAssuming you have an LDS server with a registered server\n\n```elixir\nalias OpcUA.Server\n\n#LDS Server\n{:ok, lds_pid} = Server.start_link()\n:ok = Server.set_port(lds_pid, 4050)\n:ok = Server.set_lds_config(lds_pid, \"urn:opex62541.test.local_discovery_server\")\n:ok = Server.start(lds_pid)\n\n#Normal OPC UA Server\n{:ok, s_pid} = Server.start_link()\n:ok = Server.set_port(s_pid, 4048)\n\n:ok = Server.discovery_register(s_pid,\n        application_uri: \"urn:opex62541.test.local_register_server\",\n        server_name: \"testRegister\",\n        endpoint: \"opc.tcp://localhost:4050\"\n      )\n:ok = Server.start(s_pid)\n\n# Registration time\nProcess.sleep(1500)\n```\n\n#","ref":"discovery.html#client"},{"type":"extras","title":"Scanning Network - Discovery","doc":"To discover all OPC UA Server in the network, you can use `find_servers_on_network/2`,\n\n```elixir\nalias OpcUA.Client\n\n{:ok, c_pid} = Client.start_link()\n:ok = Client.set_config(c_pid)\n\n# LDS Server url\nurl = \"opc.tcp://localhost:4050/\"\n\nClient.find_servers_on_network(c_pid, url)\n\n# this response may change depending on your computer\n{:ok,\n [\n   %{\n     \"capabilities\" => [\"LDS\"],\n     \"discovery_url\" => \"opc.tcp://localhost:4050\",\n     \"record_id\" => 0,\n     \"server_name\" => \"LDS-localhost\"\n   },\n   %{\n     \"capabilities\" => [\"NA\"],\n     \"discovery_url\" => \"opc.tcp://localhost:4048\",\n     \"record_id\" => 1,\n     \"server_name\" => \"testRegister-localhost\"\n   }\n ]}\n```\nIf you require more detailed information, you can use the `find_server/2` function.\n\n```elixir\nClient.find_servers(c_pid, url)\n\n# this response may change depending on your computer\n{:ok,\n  [\n    %{\n      \"discovery_url\" => [\"opc.tcp://localhost:4050/\"],\n      \"name\" => \"open62541-based OPC UA Application\",\n      \"product_uri\" => \"http://open62541.org\",\n      \"application_uri\" => \"urn:opex62541.test.local_discovery_server\",\n      \"server\" => \"urn:opex62541.test.local_discovery_server\",\n      \"type\" => \"discovery_server\"\n    },\n    %{\n      \"application_uri\" => \"urn:opex62541.test.local_register_server\",\n      \"discovery_url\" => [\n        \"opc.tcp://localhost:4048/\",\n        \"opc.tcp://localhost:4048/\"\n      ],\n      \"name\" => \"open62541-based OPC UA Application\",\n      \"product_uri\" => \"http://open62541.org\",\n      \"server\" => \"urn:opex62541.test.local_register_server\",\n      \"type\" => \"server\"\n    }\n  ]}\n```\n\nFinally, to get the server endpoints, you can use the `get_endpoints/2` function.\n\n```elixir\n\nurl = \"opc.tcp://localhost:4048\"\nClient.get_endpoints(c_pid, url)\n\n# this response may change depending on your computer\n{:ok,\n  [\n    %{\n      \"endpoint_url\" => \"opc.tcp://localhost:4048\",\n      \"security_level\" => 1,\n      \"security_mode\" => \"none\",\n      \"security_profile_uri\" => \"http://opcfoundation.org/UA/SecurityPolicy#None\",\n      \"transport_profile_uri\" =>\n        \"http://opcfoundation.org/UA-Profile/Transport/uatcp-uasc-uabinary\"\n    }\n  ]}\n```","ref":"discovery.html#scanning-network"},{"type":"extras","title":"Information Manipulation","doc":"# Information Manipulation\n\nThis tutorial will cover the following topics:","ref":"information-manipulation.html"},{"type":"extras","title":"Content - Information Manipulation","doc":"- [Information Model](#information-model)\n- [Server](#server)\n  - [Address Space](#address-space)\n  - [Nodes](#nodes)\n    - [Object Nodes](#object-nodes)\n    - [Variable Nodes](#variable-nodes)\n  - [Read/Write Nodes](#read/write-nodes)\n  - [Monitored Items](#monitored-items)\n  - [Write Events](#monitored-items)\n  - [Examples](#examples)\n- [Client](#client)\n  - [Read/Write Node](#read/write-node)\n  - [Monitored Item](#monitored-item)\n  - [Example](#example)\n\n#","ref":"information-manipulation.html#content"},{"type":"extras","title":"Information Model - Information Manipulation","doc":"The information model is an abstract representation of the real objects that you need to manage in your application. The same pieces of physical equipment and the associated pieces of information can be modeled in different ways according to the specific process and environment requirements.\n\nThe basic principles of information modeling in OPC UA are as follows:\n* Use of object-oriented techniques, including hierarchies and inheritance.\n* The same mechanism is used to access the types and the instances.\n* The hierarchies of the data types and the links between the nodes are extendable.\n* There are no limitations on how to model information.\n* Information modeling is always placed on the server-side.","ref":"information-manipulation.html#information-model"},{"type":"extras","title":"Server - Information Manipulation","doc":"Assuming that an OPC UA Server has been created and configured as shown in [Lifecycle](https://hexdocs.pm/opex62541/doc/lifecycle.html) tutorial.\n\n#","ref":"information-manipulation.html#server"},{"type":"extras","title":"Address Space - Information Manipulation","doc":"The set of objects and related information that an OPC UA server makes available to the clients is the address space. The address space of the OPC UA is a set of nodes that are connected by references. Each node has properties, which are called attributes. In this library, we use one namespace per address space, so we will use the namespace to refer to an address space.\n\nTo add a new namespace to your server use `add_namespace/2`:\n\n```elixir\nalias OpcUA.{NodeId, QualifiedName, Server}\n{:ok, ns_index} = Server.add_namespace(server_pid, \"Room\")\n```\n\nThese functions take a configured OPC UA Server PID and an identifier as a unique string. It will return an index to be used as a reference to create the related nodes.\n\n#","ref":"information-manipulation.html#address-space"},{"type":"extras","title":"Nodes - Information Manipulation","doc":"In OPC UA, every entity in the address space is a node. To uniquely identify a Node, each node has a NodeId; to create new NodeId's, use the [NodeId](https://hexdocs.pm/opex62541/OpcUA.NodeId.html) module, for example:\n\n```elixir\nrequested_new_node_id = NodeId.new(ns_index: 0, identifier_type: \"integer\", identifier: 1000)\n```\nIn this example, we have created a NodeId corresponding to `i=1000` (namespace index 0, numeric identifier, id 1000, according to OPC UA XML Schema)\n\n**Note**: You can use the [zero namespace index](https://github.com/OPCFoundation/UA-Nodeset/blob/v1.04/Schema/NodeIds.csv) when you are referring to builtin nodes.\n\nThere are eight standard node classes: variable, object, method, view, data type, variable type, object type, and reference type. However, for this tutorial, we will describe an example of how to define object nodes and variable nodes.\n\n**Note**: Currently, only the method node is not supported yet.\n\nFor this tutorial, we will model a temperature sensor by creating an object node of the sensor and a variable node representing the temperature.\n\n##","ref":"information-manipulation.html#nodes"},{"type":"extras","title":"Object Nodes - Information Manipulation","doc":"The object node class structures the address space and can be used to group variables, methods, or other objects.\n\n```elixir\nrequested_new_node_id = NodeId.new(ns_index: ns_index, identifier_type: \"string\", identifier: \"R1_TS1_Temperature-Sensor\")\nparent_node_id = NodeId.new(ns_index: 0, identifier_type: \"integer\", identifier: 85)\nreference_type_node_id = NodeId.new(ns_index: 0, identifier_type: \"integer\", identifier: 35)\nbrowse_name = QualifiedName.new(ns_index: ns_index, name: \"Temperature sensor\")\ntype_definition = NodeId.new(ns_index: 0, identifier_type: \"integer\", identifier: 58)\n\n:ok = Server.add_object_node(server_pid,\n  requested_new_node_id: requested_new_node_id,\n  parent_node_id: parent_node_id,\n  reference_type_node_id: reference_type_node_id,\n  browse_name: browse_name,\n  type_definition: type_definition\n)\n```\n\nFor this case, we are define an object node as a `BaseObjectType` defined by `type_definition` (wildcard for object nodes `i=58`) with a `ns=1;s=R1_TS1_Temperature-Sensor` NodeId (OPC UA XML Schema) using `requested_new_node_id`, which parent node is an `ObjectsFolder` (`parent_node_id i=85`), it uses a `Organizes`(`i=35`) relationship and the clients should see the object node as `Temperature sensor` defined by `browse_name`.\n\n**Note**: All nodes that use ns_index: 0 are [zero namespace nodes id index](https://github.com/OPCFoundation/UA-Nodeset/blob/v1.04/Schema/NodeIds.csv), which means that they are defined by OPC UA standard.\n\n##","ref":"information-manipulation.html#object-nodes"},{"type":"extras","title":"Variable Nodes - Information Manipulation","doc":"The variable node class represents a value. Clients can read, write, or subscribe to it (monitored items).\n\nNow we are going to add a variable node (to holds the `Temperature` value) to our object node just as shown in the following example:\n\n```elixir\nrequested_new_node_id = NodeId.new(ns_index: ns_index, identifier_type: \"string\", identifier: \"R1_TS1_Temperature\")\nparent_node_id = NodeId.new(ns_index: ns_index, identifier_type: \"string\", identifier: \"R1_TS1_Temperature-Sensor\")\nreference_type_node_id = NodeId.new(ns_index: 0, identifier_type: \"integer\", identifier: 47)\nbrowse_name = QualifiedName.new(ns_index: ns_index, name: \"Temperature\")\ntype_definition = NodeId.new(ns_index: 0, identifier_type: \"integer\", identifier: 63)\n\n:ok = Server.add_variable_node(server_pid,\n  requested_new_node_id: requested_new_node_id,\n  parent_node_id: parent_node_id,\n  reference_type_node_id: reference_type_node_id,\n  browse_name: browse_name,\n  type_definition: type_definition\n)\n```\n\nFor this case, we are define an object node as a `BaseDataVariableType` defined by `type_definition` (wildcard for variable nodes `i=63`) with a `ns=1;s=R1_TS1_Temperature` NodeId (OPC UA XML Schema) using `requested_new_node_id`, which parent node is the  `R1_TS1_Temperature-Sensor`defined previously, it uses a `HasComponent`(`i=47`) relationship and the clients should see the variable node as `Temperature` defined by `browse_name`.\n\n#","ref":"information-manipulation.html#variable-nodes"},{"type":"extras","title":"Read/Write Nodes - Information Manipulation","doc":"Depending on the type of node (object, variable, reference, etc.), it's [attributes](https://documentation.unified-automation.com/uasdkhp/1.0.0/html/_l2_ua_node_classes.html) can change. When the node is created, its attributes will have a default value. However, you can read and write its value using the [Server](https://hexdocs.pm/opex62541/OpcUA.Server.html) module. The following example changes the access level and the value of the node.\n\n```elixir\nnode_id = NodeId.new(ns_index: ns_index, identifier_type: \"string\", identifier: \"R1_TS1_Temperature\")\n# CurrentWrite, CurrentRead\n:ok = Server.write_node_access_level(server_pid, node_id, 3)\n{:ok, 3} = Server.read_node_access_level(server_pid, node_id)\n\n# Change the Node Value.\n:ok = Server.write_node_value(server_pid, node_id, 0, true)\n{:ok, true} = Server.read_node_value(server_pid, node_id)\n\n:ok = Server.write_node_value(server_pid, node_id, 1, 103)\n{:ok, 103} = Server.read_node_value(server_pid, node_id)\n```\n\nFirst, we set the `access_level` to `CurrentWrite` and `CurrentRead` (3). Therefore the client has access to its value. Then we change the value from a `Boolean` (0) to a `Byte` (1) value (this is possible thanks to the variable wildcard that we use before).\n\n**Note**: It is recommended to change the writing mask and the access level node attributes.\n\nThe `data_type` and `value` argument of `write_node_value/4` must be congruent, therefore, it is recommended to review the official [data types](https://opcfoundation.org/UA/schemas/1.04/Opc.Ua.Types.bsd).\n\n#","ref":"information-manipulation.html#read-write-nodes"},{"type":"extras","title":"Monitored Items - Information Manipulation","doc":"Clients can subscribe to variable nodes through `monitored items`. A monitored item is used to request an OPC UA server for notifications of each change of value in a specific node.\n\n```elixir\nnode_id = NodeId.new(ns_index: ns_index, identifier_type: \"string\", identifier: \"R1_TS1_Temperature\")\n# Add Temperature as a Monitored Item\n{:ok, 1} = Server.add_monitored_item(server_pid, monitored_item: node_id, sampling_time: 1000.0)\n# Deletes Monitored Item\n:ok = Server.delete_monitored_item(server_pid, 1)\n```\n\n#","ref":"information-manipulation.html#monitored-items"},{"type":"extras","title":"Write Event Nodes - Information Manipulation","doc":"Each time a node value is updated, the Server module sends a message to the parent process in de form of {node_id, value}\n\nTherefore, this feature can be handled by a module, as illustrated in the following example:\n\n```elixir\ndefmodule MyServer do\n  use OpcUA.Server\n  require Logger\n\n  def handle_write({node_id, value}, state) do\n    Logger.debug(\"Value changed #{value} in #{node_id} Node\")\n    state\n  end\nend\n```\n\n#","ref":"information-manipulation.html#write-event-nodes"},{"type":"extras","title":"Examples - Information Manipulation","doc":"More examples can be found in the source code [tests](https://github.com/valiot/opex62541/tree/master/test/server_tests).","ref":"information-manipulation.html#examples"},{"type":"extras","title":"Client - Information Manipulation","doc":"Assuming that an OPC UA Client has been created, configured, and connected to an OPC UA Server as shown in [Lifecycle](https://hexdocs.pm/opex62541/doc/lifecycle.html) tutorial.\n\n#","ref":"information-manipulation.html#client"},{"type":"extras","title":"Read/Write Node - Information Manipulation","doc":"You can read and write its attribute value by using the [Client](https://hexdocs.pm/opex62541/OpcUA.Client.html) module. The following example changes the value of the node.\n\n```elixir\nnode_id = NodeId.new(ns_index: ns_index, identifier_type: \"string\", identifier: \"R1_TS1_Temperature\")\n# Writes a Byte\n:ok = Client.write_node_value(client_pid, node_id, 1, 21)\n# Reads a Byte\n{:ok, 21} = Client.read_node_value(client_pid, node_id)\n```\n\n#","ref":"information-manipulation.html#read-write-node"},{"type":"extras","title":"Monitored Item - Information Manipulation","doc":"To request a monitored item to a server, the client must first request a subscription request. Then the client must request the monitored item.\n\n```elixir\nnode_id = NodeId.new(ns_index: ns_index, identifier_type: \"string\", identifier: \"R1_TS1_Temperature\")\n# Creates a subscription channels.\n{:ok, 1} = Client.add_subscription(client_pid)\n# Request a monitored item.\n{:ok, 1} = Client.add_monitored_item(client_pid, monitored_item: node_id, subscription_id: 1)\n```\n\n#","ref":"information-manipulation.html#monitored-item"},{"type":"extras","title":"Example - Information Manipulation","doc":"More examples can be found in the source code [tests](https://github.com/valiot/opex62541/tree/master/test/client_tests).","ref":"information-manipulation.html#example"},{"type":"extras","title":"Elixir Integration","doc":"# Elixir Integration","ref":"elixir-integration.html"},{"type":"extras","title":"Content - Elixir Integration","doc":"- [Integration](#integration)\n- [Server](#server)\n  - [Server Configuration](#server-configuration)\n  - [Address Space](#address-space)\n  - [Write Events](#write-events)\n  - [Examples](#examples)\n- [Client](#client)\n  - [Client Configuration](#client-configuration)\n  - [Monitored Item](#monitored-item)\n  - [Subscription](#subscription)\n  - [Example](#example)","ref":"elixir-integration.html#content"},{"type":"extras","title":"Integration - Elixir Integration","doc":"Both Server and Client modules are implemented as a `__using__` macro so that you can put it in any module;\nyou only need to add the defined callbacks to integrate this library into your project.\n\nEither way, it is always possible to directly use Server and Client modules, as shown in previous tutorials.","ref":"elixir-integration.html#integration"},{"type":"extras","title":"Server - Elixir Integration","doc":"For convenience, `Server` is a `GenServer` wrapper for automating configuration and adding the address space (information model); it also accepts the same [options](https://hexdocs.pm/elixir/GenServer.html#module-how-to-supervise) for supervision to configure the child spec and passes them along to `GenServer`, for example:\n\n```elixir\nuse OpcUA.Server, restart: :transient, shutdown: 10_000\n```\n\nWith these instructions, the Server backend will be integrated into your application module. Now you only have to add the callbacks you require.\n\nThe basic callback is the `init/2` that let you manipulate the OPC UA server by given it's PID (opc_ua_server_pid) with some user data (user_init_state):\n\n```elixir\n# Use the `init` function to configure your server.\ndef init(user_init_state, opc_ua_server_pid) do\n  # Do some initial process and start the server at your convenience.\n  Server.start(opc_ua_server_pid)\n  # You can set a new state for your app (if required).\n  user_init_state\nend\n```\n\n**Note**: No callback automatically starts the OPC UA server, so it is recommended to use `init\\2` because it is the last callback to be executed.\n\n#","ref":"elixir-integration.html#server"},{"type":"extras","title":"Server Configuration - Elixir Integration","doc":"The first executed optional callback is `configuration/1`. It gets and executes the Server configuration and discovery connection parameters as follows:\n\n```elixir\ndef configuration(_user_init_state) do\n  [\n    config: [\n      port: 4041,\n      users: [{\"alde103\", \"secret\"}]\n    ]\n  ]\nend\n```\n\nIn this example, the server will use the port `4041` with a predefined user. The `user_init_state` is propagated to this callback too.\n\n**Note**: The output of these callbacks must be a list (type config_options) as shown in the [Server](https://hexdocs.pm/opex62541/OpcUA.Server.html) module.\n\n#","ref":"elixir-integration.html#server-configuration"},{"type":"extras","title":"Address Space - Elixir Integration","doc":"The next executed optional callback is `address_space/1`. It gets and adds all nodes (namespaces, object nodes, variable nodes, monitored items) to the Server as follows:\n\n```elixir\ndef address_space(_user_init_state) do\n  [\n    namespace: \"Sensor\",\n    object_node: OpcUA.ObjectNode.new(\n      [\n        requested_new_node_id: NodeId.new(ns_index: 2, identifier_type: \"string\", identifier: \"R1_TS1_Sensor\"),\n        parent_node_id: NodeId.new(ns_index: 0, identifier_type: \"integer\", identifier: 85),\n        reference_type_node_id: NodeId.new(ns_index: 0, identifier_type: \"integer\", identifier: 35),\n        browse_name: QualifiedName.new(ns_index: 2, name: \"Temperature sensor\"),\n        type_definition: NodeId.new(ns_index: 0, identifier_type: \"integer\", identifier: 58)\n      ]\n    ),\n    variable_node: OpcUA.VariableNode.new(\n      [\n        requested_new_node_id: NodeId.new(ns_index: 2, identifier_type: \"string\", identifier: \"R1_TS1_Temperature\"),\n        parent_node_id: NodeId.new(ns_index: 2, identifier_type: \"string\", identifier: \"R1_TS1_Sensor\"),\n        reference_type_node_id: NodeId.new(ns_index: 0, identifier_type: \"integer\", identifier: 47),\n        browse_name: QualifiedName.new(ns_index: 2, name: \"Temperature\"),\n        type_definition: NodeId.new(ns_index: 0, identifier_type: \"integer\", identifier: 63)\n      ],\n      write_mask: 0x3FFFFF,\n      value: {10, 103.0},\n      access_level: 3\n    ),\n    monitored_item: OpcUA.MonitoredItem.new(\n      [\n        monitored_item: NodeId.new(ns_index: 2, identifier_type: \"string\", identifier: \"R1_TS1_Temperature\"),\n        sampling_time: 1000.0,\n        subscription_id: 1\n      ]\n    )\n  ]\nend\n```\n\nThis example emulates a temperature sensor based on an object node (`R1_TS1_Sensor`) with a variable node (`R1_TS1_Temperature`) to display the temperature, which is registered as a monitored item. The `user_init_state` is propagated to this callback too.\n\n**Note**: The output of these callbacks must be a list (type address_space_list) as shown in the [Server](https://hexdocs.pm/opex62541/OpcUA.Server.html) module. The order matters according to the node dependency. In this example, the parent node of the variable node is an object node. If the object node is not defined at the variable node definition time, the server will crash.\n\n#","ref":"elixir-integration.html#address-space"},{"type":"extras","title":"Write Events - Elixir Integration","doc":"A runtime callback is `handle_write/2`. It handles every Client writing events to any Server node, as follows:\n\n```elixir\ndef handle_write(write_event, state) do\n  # Do something with the write event ({node_id, value} = write_event)\n  # and your module state (state)\n  state\nend\n```\n\n#","ref":"elixir-integration.html#write-events"},{"type":"extras","title":"Examples - Elixir Integration","doc":"The following example shows a module that takes its configuration from the environment:\n\n```elixir\ndefmodule MyServer do\n  use OpcUA.Server\n  alias OpcUA.{NodeId, Server, QualifiedName}\n\n  # Use the `init` function to configure your server.\n  def init(user_init_state, opc_ua_server_pid) do\n    # Do some initial process and start the server at your convenience.\n    Server.start(opc_ua_server_pid)\n    # You can set a new state for your app.\n    user_init_state\n  end\n\n  def configuration(_user_init_state), do: Application.get_env(:opex62541, :configuration, [])\n  def address_space(_user_init_state), do: Application.get_env(:opex62541, :address_space, [])\n\n  def handle_write(write_event, %{parent_pid: parent_pid} = state) do\n    send(parent_pid, write_event)\n    state\n  end\nend\n```\n\nThis code can be executed as\n\n```elixir\n{:ok, my_pid} = MyServer.start_link({self(), 103} = _user_init_state)\n```\n\nMore examples can be found in the source code [tests](https://github.com/valiot/opex62541/tree/master/test/server_tests).","ref":"elixir-integration.html#examples"},{"type":"extras","title":"Client - Elixir Integration","doc":"The `Client` module can be initialized manually (as shown in previous tutorials) or by overwriting `configuration/1` and `monitored_items/1` callbacks to auto-set the configuration and subscription items. It also helps you handle the Client's \"subscription\" events (monitorItems) by overwriting `handle_subscription/2` callback.\n\nLike the `Server` module, the `Client` module is also based on a `GenServer` behavior. Therefore it accepts the same [options](https://hexdocs.pm/elixir/GenServer.html#module-how-to-supervise) for supervision to configure the child spec and passes them along to `GenServer`; to add the `Client` behavior to your application, use the following code:\n\n```elixir\n  use OpcUA.Client, restart: :transient, shutdown: 10_000\n```\n\nThe basic callback is the `init/2` that let you maniplulate the OPC UA client by given its PID (opc_ua_server_pid) with some user data (user_init_state):\n\n```elixir\n# Use the `init` function to configure your client.\ndef init({parent_pid, 103} = _user_init_state, opc_ua_client_pid) do\n  # this will be your app state\n  %{parent_pid: parent_pid, opc_ua_client_pid: opc_ua_client_pid}\nend\n```\n\n#","ref":"elixir-integration.html#client"},{"type":"extras","title":"Client Configuration - Elixir Integration","doc":"The first executed optional callback is `configuration/1`, it gets and execute the Client configuration and handles server connection parameters as follows:\n\n```elixir\ndef configuration(_user_init_state) do\n  {:ok, localhost} = :inet.gethostname()\n  [\n    config: [\n      set_config: %{\n        \"requestedSessionTimeout\" => 1200000,\n        \"secureChannelLifeTime\" => 600000,\n        \"timeout\" => 50000\n      }\n    ],\n    conn: [\n      by_username: [\n        url: \"opc.tcp://#{localhost}:4041/\",\n        user: \"alde103\",\n        password: \"secret\"\n      ]\n    ]\n  ]\nend\n```\n\nIn this example, the client will use a predefined timeout and connection (url, user, etc.) parameters. The `user_init_state` is propagated to this callback too.\n\n**Note**: The output of these callbacks must be a list (type config_options) as shown in the [Client](https://hexdocs.pm/opex62541/OpcUA.Client.html) module.\n\n#","ref":"elixir-integration.html#client-configuration"},{"type":"extras","title":"Monitored Item - Elixir Integration","doc":"The next optional callback to be executed is `monitored_items/1`. It gets and adds all nodes (namespaces, object nodes, variable nodes, monitored items) to the Server, for example:\n\n```elixir\ndef monitored_items(_user_init_state) do\n  [\n    subscription: 200.0,\n    monitored_item: OpcUA.MonitoredItem.new(\n      [\n        monitored_item: NodeId.new(ns_index: 2, identifier_type: \"string\", identifier: \"R1_TS1_Temperature\"),\n        sampling_time: 100.0,\n        subscription_id: 1\n      ]\n    )\n  ]\nend\n```\n\nIn this example, the client automatically sends a subscription request (with a publishing interval of 200.0 ms) and a monitored item request (`R1_TS1_Temperature`), the `user_init_state` is propagated to this callback too.\n\n**Note**: The output of these callbacks must be a list (type address_space_list) as shown in the [Client](https://hexdocs.pm/opex62541/OpcUA.Client.html) module.\n\n\nAn runtime callback is `handle_monitored_data/2`, it handles every Server events triggered by a monitored item data, as follows:\n\n```elixir\ndef handle_monitored_data(monitored_item_event, state) do\n  # Do something with the event ({subscription_id, monitored_id, value} = monitored_item_event)\n  # and your module state (state)\n  state\nend\n```\n\nAnother runtime callback is `handle_deleted_monitored_item/2`, it handles the withdrawal events of monitored items from the server, for example:\n\n```elixir\ndef handle_deleted_monitored_item(subscription_id, monitored_id, state) do\n  # Do something with this event ({subscription_id, monitored_id, value} = monitored_item_event)\n  # and your module state (state)\n  state\nend\n```\n\n#","ref":"elixir-integration.html#monitored-item"},{"type":"extras","title":"Subscription - Elixir Integration","doc":"The library automatically creates subscriptions. However, there are some callbacks to handle unexpected events, as shown below:\n\n```elixir\ndef handle_subscription_timeout(subscription_id, state) do\n  # Do something with this event (subscription_id is an integer)\n  # and your module state (state)\n  state\nend\n```\n\nThe `handle_deleted_subscription/2` handles the withdrawal events of subscriptions from the server, for example:\n\n```elixir\ndef handle_deleted_subscription(subscription_id, state) do\n  # Do something with this event (subscription_id is an integer)\n  # and your module state (state)\n  state\nend\n```\n\n\n#","ref":"elixir-integration.html#subscription"},{"type":"extras","title":"Example - Elixir Integration","doc":"The following example shows a module that takes its configuration from the environment and notifies any client event to its parent process:\n\n```elixir\ndefmodule MyClient do\n  use OpcUA.Client\n  alias OpcUA.Client\n\n  # Use the `init` function to configure your Client.\n  def init({parent_pid, 103} = _user_init_state, opc_ua_client_pid) do\n    %{parent_pid: parent_pid, opc_ua_client_pid: opc_ua_client_pid}\n  end\n\n  def configuration(_user_init_state), do: Application.get_env(:my_client, :configuration, [])\n  def monitored_items(_user_init_state), do: Application.get_env(:my_client, :monitored_items, [])\n\n  def handle_subscription_timeout(subscription_id, state) do\n    send(state.parent_pid, {:subscription_timeout, subscription_id})\n    state\n  end\n\n  def handle_deleted_subscription(subscription_id, state) do\n    send(state.parent_pid, {:subscription_delete, subscription_id})\n    state\n  end\n\n  def handle_monitored_data(changed_data_event, state) do\n    send(state.parent_pid, {:value_changed, changed_data_event})\n    state\n  end\n\n  def handle_deleted_monitored_item(subscription_id, monitored_id, state) do\n    send(state.parent_pid, {:item_deleted, {subscription_id, monitored_id}})\n    state\n  end\n\n  def read_node_value(pid, node), do: GenServer.call(pid, {:read, node}, :infinity)\n\n  def get_client(pid), do: GenServer.call(pid, {:get_client, nil})\n\n  def handle_call({:read, node}, _from, state) do\n    resp = Client.read_node_value(state.opc_ua_client_pid, node)\n    {:reply, resp, state}\n  end\n\n  def handle_call({:get_client, nil}, _from, state) do\n    {:reply, state.opc_ua_client_pid, state}\n  end\nend\n```\nThis code can be executed as\n\n```elixir\n{:ok, c_pid} = MyClient.start_link({self(), 103})\n```\n\nMore examples can be found in the source code [tests](https://github.com/valiot/opex62541/tree/master/test/client_tests).","ref":"elixir-integration.html#example"}],"content_type":"text/markdown"}